package org.objectweb.kilim;

import org.objectweb.kilim.model.instanciation.DefaultInstanciationStrategy;
import org.objectweb.kilim.model.instanciation.InstanciationStrategy;
import org.objectweb.kilim.model.mapping.DefaultMappingContext;
import org.objectweb.kilim.model.mapping.DefaultMappingStrategy;
import org.objectweb.kilim.model.mapping.MappingContext;
import org.objectweb.kilim.model.mapping.MappingStrategy;
import org.objectweb.kilim.model.services.Annotations;
import org.objectweb.kilim.model.services.ExternalValueReferences;
import org.objectweb.kilim.repository.ClassLoaderResourceLoader;
import org.objectweb.kilim.repository.Repository;
import org.objectweb.kilim.repository.ResourceLoader;
import org.objectweb.kilim.repository.ResourceRepository;
import org.objectweb.kilim.repository.TemplateDescriptionParser;

/**
 * A class where all the factories and global tables used by the Kilim environment, are created.
 * @author Horn
 */
public class KilimConfiguration extends Annotations {
	private static TemplateDescriptionParser parser;
	private static Repository repository;
	private static ResourceLoader loader;
	private static InstanciationStrategy instanciationStgy;
	private static ExternalValueReferences xReferences;
	private static MappingStrategy mappingStgy;
	private static MappingContext mappingContext;

	static {	
		parser = new TemplateDescriptionParser(true);
		repository = new ResourceRepository(parser);
		loader = new ClassLoaderResourceLoader(repository.getClass().getClassLoader());
		repository.setResourceLoader(loader);					
		try {
			instanciationStgy = new DefaultInstanciationStrategy();
		} catch (KilimException ex) {
			System.out.println("Problem when instanciating the default instanciation strategy");
			System.exit(1);
		}
		try {
			mappingStgy = new DefaultMappingStrategy();
			mappingContext = new DefaultMappingContext();
		} catch (KilimException ex) {
			System.out.println("Problem when instanciating the default mapping strategy");
			System.exit(2);
		}
	}
	
	/**
	 * Method getTemplateDescriptionParser. This method returns the SAX parser used to parse the templates stored in .kilim files
	 * @return TemplateDescriptionParser
	 */
	public static TemplateDescriptionParser getTemplateDescriptionParser() {
		return parser;
	}
	
	/**
	 * Method setTemplateDescriptionParser. This method sets the SAX parser used to parse the templates stored in .kilim files
	 * @param aParser :
	 */
	public static void setTemplateDescriptionParser(TemplateDescriptionParser aParser) {
		parser = aParser;
	}

	/**
	 * Method getInstanciationStrategy. An instanciation strategy defines the instanciation managers to be used for each component of the system.
	 * This method returns the instanciation strategy.
	 * @return InstanciationStrategy
	 */
	public static InstanciationStrategy getInstanciationStrategy() {
		return instanciationStgy;
	}

	/**
	 * Method getLoader.  This method returns the resource loader used to get the .kilim files.
	 * @return ResourceLoader
	 */
	public static ResourceLoader getLoader() {
		return loader;
	}

	/**
	 * Method getMappingStrategy. A mapping strategy defines the mappers used to effectively perform the mechanism associated to each Runtime source and action.
	 * This method returns the mapping strategy.
	 * @return MappingStrategy
	 */
	public static MappingStrategy getMappingStrategy() {
		return mappingStgy;
	}

	/**
	 * Method getMappingContext. A mapping strategy is enforced through mappers that take a global and shared context as a parameter.
	 * This method returns the mapping context.
	 * @return MappingContext
	 */
	public static MappingContext getMappingContext() {
		return mappingContext;
	}

	/**
	 * Method getRepository. The parsers are seen in Kilim as specific implementation of a generic Repository interface
	 * (@see org.objectweb.kilim.repository.Repository). This method returns the repository.
	 * @return Repository
	 */
	public static Repository getRepository() {
		return repository;
	}

	/**
	 * Method setInstanciationStgy. This method sets the instanciation strategy (@see org.objectweb.kilim.model.instanciation.InstanciationStrategy)
	 * @param aStrategy : the strategy to be set.
	 */
	public static void setInstanciationStrategy(InstanciationStrategy aStrategy) {
		instanciationStgy = aStrategy;
	}

	/**
	 * Method setLoader. This method sets the resource loader used to get the .kilim files.
	 * @param aLoader : the loader.
	 */
	public static void setLoader(ResourceLoader aLoader) {
		loader = aLoader;
	}

	/**
	 * Method setMappingStrategy.
	 * @param aStrategy : the strategy.
	 */
	public static void setMappingStrategy(MappingStrategy aStrategy) {
		mappingStgy = aStrategy;
	}
	
	/**
	 * Method setMappingContext.
	 * @param aContext : the mapping context.
	 */
	public static void setMappingContext(MappingContext aContext) {
		mappingContext = aContext;
	}

	/**
	 * Sets the repository.
	 * @param repository The repository
	 */
	public static void setRepository(Repository repository) {
		KilimConfiguration.repository = repository;
	}
	/**
	 * Returns the xReferences.
	 * @return ExternalValueReferences
	 */
	public static ExternalValueReferences getExternalReferences() {
		return xReferences;
	}

	/**
	 * Sets the xReferences.
	 * @param xRefs The xReferences to set
	 */
	public static void setExternalReferences(ExternalValueReferences xRefs) {
		xReferences = xRefs;
	}
}
