/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * This class describes the binding between to bindable elements. Initially limited to describe bindings between ports, it has been extended to allow
 * the binding of a port to any kind of value sources.
 */
public class Binding extends TemplateElementImpl {
	private String portName;
	private List boundProviders;
	
	/**
	 * The public constructor of a binding object.
	 * @param aPort : reference of the port to be bound.
	 * @param aTemplate : reference of the containing template. 
	 * @throws KilimException : generated when the port reference is null or when the template reference is null.
	 */
	public Binding(String aPort, TemplateDescription  aTemplate) throws KilimException {
		if (aPort == null) {
			if (aTemplate != null) {
				throw new KilimException("attempt to use a null name for a port in a bind in template " + aTemplate.getName());
			} else {
				throw new KilimException("attempt to use a null name for a port in a bind in a null template " + aTemplate.getName());
			}	
		}
		if (aTemplate == null) {
			throw new KilimException("attempt to bind port " + aPort + " in a null template ");
		}
		portName = aPort;
		setContainingTemplate(aTemplate);
	}
		
	/**
	 * returns the name of the bound port.
	 * @return String
	 */
	public String getPortName() {
		return portName;
	}

	/**
	 * sets the name of the bound port
	 * @param aPortName : the port name
	 */
	public void setPortName(String aPortName) {
		portName = aPortName;
	}
	
	/**
	 * returns as an iterator the bound target providers. The iterator contains at most one element if the port is unary.
	 * It may contain several providers if the port is nary.
	 * @return Iterator : it returns KILIM.EMPTY_ITERATOR if the port is not bound to any source.
	 */
	public Iterator getBoundProviders() {
		if (boundProviders == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return boundProviders.listIterator();
	}
	
	/**
	 * adds the target bound port.
	 * @param aElement : the value source the port  should be bound to.
	 * @throws KilimException : generated when the element is null.
	 */
	public void bindProvider(BasicElement aElement) throws KilimException {
		if (aElement == null) {
			throw new KilimException("illegal null element in binding port " + portName);
		}
		if (boundProviders == null) {
			boundProviders = new ArrayList();
		}
		boundProviders.add(aElement);
	}

	/**
	 * removes a previously bound source.
	 * @param aElement : the element to unbind.
	 * @throws KilimException : generated when the reference of the element is null or when the binding is empty.
	 */
	public void unbindProvider(BasicElement aElement) throws KilimException {
		if (aElement == null) {
			throw new KilimException("illegal null element in unbinding port " + portName + " in template " + getContainingTemplate().getName());
		}
		
		if (boundProviders == null) {
			throw new KilimException("attempt to remove an element from an empty binding " + portName + " in template " + getContainingTemplate().getName());
		}
		boundProviders.remove(aElement);
	}
}
