/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public class Instance extends TemplateElementImpl implements NamedElement {
	private String localName;
	private int status;		
	private TemplateDescription template;

    /** Constructs a new Instance in a template
	 * @param aName identifies the Instance in its Template
	 * @param aStatus :indicates the status of the template 
	 * (should be one of KILIM.PUBLIC, KILIM.PROTECTED, KILIM.PRIVATE)
	 * @param aTemplate :indicates the templates in which the template is defined.
	 * @param aContain :
	 * @throws KilimException : thrown when the name is null or the statu illegal.
	 */
	public Instance(String aName, int aStatus, TemplateDescription aTemplate, TemplateDescription aContain) throws KilimException {
		if (aName == null) {
			throw new KilimException("illegal null name for a named template element in template " + getContainingTemplate().getName());
		}
		checkStatus(aStatus);
		localName = aName;
		status = aStatus;
		template = aTemplate;
		setContainingTemplate(aContain);
	}
	
	/**
	 * @see org.objectweb.kilim.description.NamedElement#setLocalName(String)
	 */
	public void setLocalName(String aName) throws KilimException {
		String oName = getLocalName();
		if (aName == null) {
			throw new KilimException("null name in setting local name of a component " + oName + " in template " + getContainingTemplate().getName());
		}
		localName = aName;
		TemplateDescription template = getContainingTemplate();
		if (template != null) {
			template.removeLocalInstance(oName);
			template.addLocalInstance(this);
		}
	}
	
	/**
	 * @see org.objectweb.kilim.description.NamedElement#getLocalName()
	 */
	public String getLocalName() {
		return localName;
	}
	
	/**
	 * @see org.objectweb.kilim.description.NamedElement#getStatus()
	 */
	public int getStatus() {
		return status;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setStatus(int)
	 */
	public void setStatus(int aStatus) throws KilimException {
		checkStatus(aStatus);
		status = aStatus;
		TemplateDescription template = getContainingTemplate();
		if (template != null) {
			template.removeLocalInstance(getLocalName());
			template.addLocalInstance(this);
		}
	}
	
	/**
	 * returns the template describing the instance.
	 * @return String
	 */
	public TemplateDescription getTemplate() {
		return template;
	}

	/**
	 * set the template describing the instance.
	 * @param aTemplate :
	 */
	public void setTemplate(TemplateDescription aTemplate) {
		template = aTemplate;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "[" + getLocalName() + "]";
	}
	
	private void checkStatus(int aStatus) throws KilimException {
		if (aStatus < 1 || aStatus > 3) {
			throw new KilimException("illegal value " + aStatus + " for element " + localName);
		}
	}
}