/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * Describes a Kilim Template Property.
 * Provides the basic primitive types & their values.
 */
public class Property extends BasicNamedElementImpl {
	private static final Class[] TYPE_CLASSES = {
		null,
		Boolean.class,
		Character.class,
		Byte.class,
		Short.class,
		Integer.class,
		Long.class,
		Float.class,
		Double.class,
		String.class,
		Object.class
	};
				
	private Class type;
	private Object value = null;
	private int typeKind;
	private boolean isBound;
	
	/**
	 * The public constructor for properties.
	 * @param aName : the name of the property.
	 * @param aStatus : the status of the property. It should be one KILIM.PUBLIC, PUBLIC.PROTECTED, KILIM.PRIVATE.
	 * @param aTypeKind : the type kind of the property. It should be one of the kind defined in the class KILIM.
	 * @param aTemplate : the template in which the property is defined.
	 * @throws KilimException : generated if aName or a Template is null, if aStatus or aTypeKind has an illegal value.
	 */
	public Property(String aName, int aStatus, int aTypeKind, TemplateDescription aTemplate) throws KilimException {
		super(aName, aStatus, true, false, aTemplate);
		type = getClassFromTypeKind(aTypeKind);
		typeKind = aTypeKind;
	}
	
	/** 
	 * sets the property type
	 * @param aType : a type kind defined in the KILIM class.
	 * @throws KilimException : generated if Type has an illegal value.
	 */
	public void setType(int aType) throws KilimException {
		type = getClassFromTypeKind(aType);
		typeKind = aType;
	}
	
	/**
	 * returns the property type.
	 * @return Class
	 */
	public Class getType() {
		return type;
	}
	
	/**
	 * @see org.objectweb.kilim.description.Property#getTypeKind()
	 */
	public int getTypeKind() {
		return typeKind;
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.PROPERTY;
	}
		
	/**
	 * @see org.objectweb.kilim.description.Property#setValue(Object)
	 */
	public void setValue(Object aObject) throws KilimException {
		if (aObject == null) {
			value = null;
			isBound = false;
		}
		
		if (!aObject.getClass().isAssignableFrom(type)) {
			throw new KilimException("Property Exception : trying to assign " + aObject + " of type [" + aObject.getClass() + "] to type " + type);
		}
		
		isBound = true; 
		value = aObject;
	}
		
	/**
	 * @see org.objectweb.kilim.description.Property#getValue()
	 */
	public Object getValue() {
		return value;
	}
	
	private static Class getClassFromTypeKind(int aTypeKind) throws KilimException {
		if (aTypeKind < 0 || aTypeKind > 9) {
			throw new KilimException("illegal value for typeKind :  should be between 1 and 9");
		}
		return TYPE_CLASSES[aTypeKind];
	}

}
