/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * Describes a slot, which is just a set of unbound ports
 */
public class Slot extends TemplateElementImpl implements NamedElement {
	
	private static String[][] msgTexts = {
		{ "attempt to create a slot with a null name in template ", "" },	//0
		{ "attempt to create a slot ", " in a null template" },					//1
		{ "attempt to set a null name to a slot in template ", "" },			//2
		{ "attempt to use an undefined  status ", " for a slot " },			//3
		{ "attempt to set a null name to a slot in template ", "" },			//4
		{ "attempt ot add a null port in slot ", "" },								//5
		{ "attempt to add an already added port ", " in slot " },			//6
		{ "attempt to remove a null port from a slot ", "" },					//7
		{ "attempt to remove a port from an empty slot ", "" },				//8
		{ "attempt to add a null binding frin slot ", "" },							//9
		{ "attempt to remove a null binding from a slot ", "" },				//10
		{ "attempt to add an already added binding ", " in slot " },		//11
		{ "attempt to remove a binding from an empty slot ", "" }			//12
	};	
		
	private String slotName;
	private int status;
	private List ports;
	
	/**
	 * a public constructor for slots.
	 * @param aSlotName : the slot name
	 * @param aStatus : the status of the slot. It should be one KILIM.PRIVATE, KILIM.PROTECTED, KILIM.PUBLIC.
	 * @param aTemplate : the template in which the slot is defined.
	 * @throws KilimException : generated if aSlotName or aTemplate is null or if aStatus has an invalid value.
	 */
	public Slot(String aSlotName, int aStatus, TemplateDescription aTemplate) throws KilimException {
		if (aSlotName == null) {
			throw new KilimException(msgTexts[0][0] + aTemplate);
		}
		if (aTemplate == null) {
			throw new KilimException(msgTexts[1][0] + aSlotName + msgTexts[1][1]);
		}
		checkStatus(aStatus);	
		slotName = aSlotName;
		status = aStatus;
		setContainingTemplate(aTemplate);
	}
		
	/**
	 * @see org.objectweb.kilim.description.NamedElement#getLocalName()
	 */
	public String getLocalName() {
		return slotName;
	}

	/**
	 * @see org.objectweb.kilim.description.NamedElement#setLocalName(String)
	 */
	public void setLocalName(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException(msgTexts[2][0] + getContainingTemplate());
		}

		slotName = aName;
	}
	
	/**
	 * @see org.objectweb.kilim.description.NamedElement#getStatus()
	 */
	public int getStatus() {
		return status;
	}

	/**
	 * @see org.objectweb.kilim.description.NamedElement#setStatus(int)
	 */
	public void setStatus(int aStatus) throws KilimException {
		checkStatus(aStatus);
		status = aStatus;
	}
		
	/**
	 * Method checkStatus.
	 * @param aStatus
	 * @throws KilimException
	 */
	private void checkStatus(int aStatus) throws KilimException {
		if (aStatus < 1 || aStatus > 3) {
			throw new KilimException(msgTexts[3][0] + aStatus + msgTexts[3][1] + msgSuffix1());
		}
	}

	/**
	 * Method getPorts returns as an iterator the ports defined in the slot.
	 * @return Iterator
	 */
	public Iterator getPorts() {
		if (ports == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return ports.listIterator();
	}
	
	/**
	 * Method addPort adds a new port in the slot.
	 * @param aPort : the port to be added.
	 * @throws KilimException : generated if aPort is null or if the slot already contains aPort.
	 */
	public void addPort(Port aPort) throws KilimException {
		if (aPort == null) {
			throw new KilimException(msgTexts[4][0] + msgSuffix1());
		}
		if (ports == null) {
			ports = new ArrayList();
		}
		
		if (ports.contains(aPort)) {
			throw new KilimException(msgTexts[6][0] + aPort.getLocalName() + msgTexts[6][1] + msgSuffix1());
		}
		ports.add(aPort);
	}
	
	/**
	 * Method removePort.
	 * @param aPort : the port to be removed.
	 * @throws KilimException : generated if aPort is null or if no port have been defined in the slot.
	 */
	public void removePort(Port aPort) throws KilimException {
		if (aPort == null) {
			throw new KilimException(msgTexts[7][0] + msgSuffix1());
		}
		
		if (ports == null) {
			throw new KilimException(msgTexts[8][0] + msgSuffix1());
		}
		ports.remove(aPort);
	}
		
	private String msgSuffix1() {
		return slotName + " in template " + getContainingTemplate().getName();
	}
}