package org.objectweb.kilim.description;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */

public class TpMethod extends InlinedElement {
	private BasicElement support;
	private String methodName;
	private boolean isStatic;

	private List parameters;
	private Object resultValue;

	/**
	 * public constructor of a descriptor for a method used in a kilim template.
	 * @param aSupport : the element on which the method will be invoked.
	 * @param aMethodName : the name of the method.
	 * @param jStatic : true if the method is static.
	 * @param isP : true if the method can be used as a provider.
	 * @param isT : true if the method can be used as a transformer.
	 * @param aTemplate : the template in which the method is declared.
	 * @throws KilimException : generated if aSupport, aMethodName or aTemplate is null.
	 */
	public TpMethod(BasicElement aSupport, String aMethodName, boolean jStatic, boolean isP, boolean isT, TemplateDescription aTemplate) throws KilimException {
		super(isP, isT, aTemplate);
		if (aMethodName == null) {
			throw new KilimException("illegal null method name in MethodActionImpl");
		}

		support = aSupport;
		methodName = aMethodName;
		isStatic = jStatic;
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.METHOD;
	}

	/**,
	 * @see org.objectweb.kilim.description.libs.MethodAction#isStatic()
	 */
	public boolean isStatic() {
		return isStatic;
	}
	
	/**
	 * @see org.objectweb.kilim.description.libs.FieldAction#isStatic(boolean)
	 */
	public void isStatic(boolean jStatic) throws KilimException {
		isStatic = jStatic;
	}
	
	/**
	 * @see org.objectweb.kilim.description.libs.MethodAction#getSupport()
	 */
	public BasicElement getSupport() {
		return support;
	}
	
	/**
	 * sets the support of the method i.e. the object on which the method will be invoked.
	 * @param aSupport : the element to be used as a support.
	 * @throws KilimException ; generated if aSupport is null
	 */
	public void setSupport(BasicElement aSupport) throws KilimException {
		if (aSupport == null) {
			throw new KilimException("no null support is allowed in method source ");
		}
		support = aSupport;
	}
	
	/**
	 * returns the method name.
	 * @return String
	 */
	public String getMethodName() {
		return methodName;
	}
	
	/**
	 * sets the method name.
	 * @return String
	 */
	public void setMethodName(String aMethodName) throws KilimException {
		if (aMethodName == null) {
			throw new KilimException("no null method name is allowed in method source ");
		}
		methodName = aMethodName;
	}

	/**
	 * returns the parameter number.
	 * @return int
	 */
	public int getParameterNumber() {
		if (parameters == null) {
			return 0;
		}
		return parameters.size();
	}
			

	/**
	 * adds a new parameter.
	 * @param aParam : the parameter to be added
	 * @throws KilimException : generated if aParam is null or if if its name clashes with an existing one.
	 */
	public  void addParameter(Parameter aParam) throws KilimException {
		if (aParam == null) {
			throw new KilimException("illegal null value when adding parameter");
		}
		if (aParam.isEventSource()) {
			if (parameters == null) {
				parameters = new ArrayList();
			}
			parameters.add(aParam);
			return;
		}
		if (checkParameterUnicity(aParam)) {
			if (parameters == null) {
				parameters = new ArrayList();
			}
			parameters.add(aParam);
		} else {
			throw new KilimException("name clash in parameter addition	: " + aParam.getName() + " of method " + methodName + " in template " + getContainingTemplate().getName());
		}
	}
	

	/**
	 * inserts a parameter at a given position.
	 * @param aParam : the parameter to be inserted.
	 * @param position : the position
	 * @throws KilimException : generated if aParam is null, if its name clashes with existing parameters
	 */
	public  void insertParameter(Parameter aParam, int position) throws KilimException {
		if (aParam == null) {
			throw new KilimException("illegal null value when inserting parameter to method " + methodName + " in template " + getContainingTemplate().getName());
		}
		if (aParam.isEventSource()) {
			if (parameters == null) {
				parameters = new ArrayList();
			}
			parameters.add(position, aParam);
			return;
		}
		if (checkParameterUnicity(aParam)) {
			if (parameters == null) {
				parameters = new ArrayList();
			}
			parameters.add(position, aParam);
		} else {
			throw new KilimException("name clash in parameter insertion	: " + aParam.getName() + "in method " + methodName + " in template " + getContainingTemplate().getName());
		}
	}

	/**
	 * removes a parameter.
	 * @param aParam : the parameter to be removed.
	 * @throws KilimException : generated if aParam is null or is unknown
	 */
	public void removeParameter(Parameter aParam) throws KilimException {
		if (aParam == null) {
			throw new KilimException("illegal null parameter in parameter removal in method " + methodName + " in template " + getContainingTemplate().getName());
		}
		if (parameters == null) {
			throw new KilimException("illegal parameter removal on no arg method " + methodName + " in template " + getContainingTemplate().getName());
		}		
		boolean result = parameters.remove(aParam);
		if (!result) {
			throw new KilimException("attempt to removel an unknown parameter in method " + methodName + " in template " + getContainingTemplate().getName());
		}
	}

	/**
	 * returns the parameter of a given position.
	 * @param aPosition : the position
	 * @return Parameter
	 * @throws KilimException : generated if the position has an illegal value.
	 */
	public Parameter getParameter(int aPosition) throws KilimException {
		if (parameters == null) {
			throw new KilimException("illegal getParameter on no arg method " + methodName + " in template " + getContainingTemplate().getName());
		}
		if (aPosition < 0 || aPosition >= parameters.size()) {
			throw new KilimException("illegal position value in getParameter of method " + methodName + " in template " + getContainingTemplate().getName());
		}
		return (Parameter) parameters.get(aPosition);
	}
	

	/**
	 * returns as an iterator the parameters of the method.
	 * @return Iterator
	 */
	public Iterator getParameters() {
		if (parameters == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		
		return parameters.listIterator();
	}
	
	private boolean checkParameterUnicity(Parameter aParam) throws KilimException {
		if (parameters == null) {
			return true;		
		}
		
		if (aParam.isEventSource()) {
			return true;
		}
		String name = aParam.getName();
		if (name == null) {
			return true;
		}
		int pLength = parameters.size();	
		for (int i = 0; i < pLength ; i++) {
			if (name != null && name.equals(((Parameter) parameters.get(i)).getName())) {
				return false;
			}
		}
		return true;
	}
}