/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.lang.reflect.Array;

import java.util.LinkedHashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Stack;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimConfiguration;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.ArraySource;
import org.objectweb.kilim.description.BasicElement;
import org.objectweb.kilim.description.BasicNamedElement;
import org.objectweb.kilim.description.Binding;
import org.objectweb.kilim.description.ClassSource;
import org.objectweb.kilim.description.EventSource;
import org.objectweb.kilim.description.Instance;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.NullElement;
import org.objectweb.kilim.description.Plug;
import org.objectweb.kilim.description.Port;
import org.objectweb.kilim.description.Property;
import org.objectweb.kilim.description.Provider;
import org.objectweb.kilim.description.Reference;
import org.objectweb.kilim.description.Slot;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.description.TemplateElementImpl;
import org.objectweb.kilim.description.TpAccessor;
import org.objectweb.kilim.description.TpConstructor;
import org.objectweb.kilim.description.TpGetter;
import org.objectweb.kilim.description.TpMethod;
import org.objectweb.kilim.description.TpSetter;
import org.objectweb.kilim.description.Transformer;
import org.objectweb.kilim.description.Trigger;

import org.objectweb.kilim.model.instanciation.InstanciationStrategy;

import org.objectweb.kilim.model.mapping.Mapper;
import org.objectweb.kilim.model.mapping.MappingContext;

import org.objectweb.kilim.model.services.DefaultNamingContext;
import org.objectweb.kilim.model.services.DefaultRuntimeClassLoader;

/**
 * @author horn
 */
public class ComponentFactory extends DefaultNamingContext implements Factory {	
	private String localName;	
	private ComponentFactory containingFactory;	
	private List subFactories;
	
	private TemplateDescription currentTemplate;
	private TemplateDescription containingTemplate;
	
	private RtComponent currentComponent;		
	private RtComponent containingComponent;

	private LinkedHashMap references;		
	
	/* ************************************************************************************/
	//inner class for a runtime descriptor of a getter
	
	private static class RTGetter extends RtComponentProvider {
						
		RTGetter(TpGetter aGetter, Component aComponent, RuntimeSource aSupport) {
			super(aGetter, aComponent, aSupport);
		}
					
		String getFieldName() {
			return ((TpAccessor) getElementDescription()).getFieldName();
		}
		
		boolean isStatic() {
			return ((TpAccessor) getElementDescription()).isStatic();
		}
				
		public Object specificGetValue() throws KilimException {
			if (mappingContext != null) {
				mappingContext.getContextStack().push(this);
			}
			mapper.enterContext(mappingContext);
			
			Object eventSrcValue = getEventSourceValue();			
			
			//specific part of the getValue
			RuntimeSource support = (RuntimeSource) getSupport();						
			if (support == null) { 
				throw  new KilimException("attempt to get the value of a non initialized getter");
			}
			
			Object suppObject = null;			
			if (support instanceof RTEventSource) {
				suppObject = eventSrcValue;
			} else {
				suppObject = support.getValue();
			}
			
			String fieldName = getFieldName();
			
			//call to the mapper that performs the real task
			if (mappingContext != null) {
				mappingContext.getCallStack().push(this);
			}

			Object resultValue = mapper.getGetterValue(suppObject, isStatic(), fieldName, mappingContext);

			if (mappingContext != null) {
				mappingContext.getCallStack().pop();
			}
			mapper.leaveContext(mappingContext);
			
			return resultValue;
		}
		
		public boolean hasValue() throws KilimException {
			return getSupport().hasValue();
		}
		
		public	boolean checkValue(Stack exclude) throws KilimException {
			RuntimeSource lSupport = (RuntimeSource) getSupport().getTarget();

			if (exclude == null || lSupport.hasValue()) {
				return true;
			}
			
			int eSize = exclude.size();
			for (int i = 0; i < eSize; i++) {
				if (lSupport == exclude.get(i)) {
					return false;
				}
			}

			exclude.push(this);
			boolean isOK = lSupport.checkValue(exclude);
			exclude.pop();
			return isOK;
		}
	}

	/* ************************************************************************************/
	//inner class for a runtime descriptor of a setter
	
	private static class RTSetter extends RtComponentElement implements RuntimeAction {
		private String name;
		private Object source;
		private RuntimeSource value;
		private RuntimeSource support;
		private Mapper mapper;
		private MappingContext mappingContext;
				
		RTSetter(TpSetter aGetter, Component aComponent, RuntimeSource aSupport, RuntimeSource aValue) {
			super(aGetter, aComponent);
			mapper = KilimConfiguration.getMappingStrategy().getDefaultMapper();
			mappingContext = KilimConfiguration.getMappingContext();
			support = aSupport;
			value = aValue;
		}
		
		/**
	 	* @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 	*/
		public String getLocalName() {
			return name;
		}
		
		public void setEventSourceValue(Object aValue) {
			source = aValue;
		}

		public Object getEventSourceValue() throws KilimException {
			return source;
		}
		
		String getFieldName() {
			return ((TpAccessor) getElementDescription()).getFieldName();
		}
		
		boolean isStatic() {
			return ((TpAccessor) getElementDescription()).isStatic();
		}	
	
		public void execute() throws KilimException {
			if (support == null) { 
				throw  new KilimException("attempt to get the value of a non initialized setter");
			}

			if (mappingContext != null) {
				mappingContext.getCallStack().push(this);
			}
			mapper.enterContext(mappingContext);
			
			Object suppObject = null;
			
			if (support instanceof RTEventSource) {
				suppObject = ((RuntimeSource) source).getEventSourceValue();
			} else {
				suppObject = support.getValue();
			}
			
			String fieldName = getFieldName();

			Object toBeSet = null;			
			if (value instanceof RTEventSource) {
				toBeSet = source;
			} else {
				toBeSet = value.getValue();
			}

			mapper.executeSetter(suppObject, isStatic(), fieldName, toBeSet, mappingContext);
			if (mappingContext != null) {
				mappingContext.getCallStack().pop();
			}
			mapper.leaveContext(mappingContext);
		}
		
		public boolean checkAction(Stack exclude) throws KilimException {
			RuntimeSource lSupport = (RuntimeSource) support.getTarget();
			if (exclude == null || lSupport.hasValue()) {
				return true;
			}
			
			int eSize = exclude.size();
			for (int i = 0; i < eSize; i++) {
				if (lSupport == exclude.get(i)) {
					return false;
				}
			}

			exclude.push(this);
			boolean isOK = lSupport.checkValue(exclude);
			exclude.pop();
			return isOK;
		}
	}
	
	/* ************************************************************************************/
		//inner class for a runtime descriptor of an array
		
	private static class RTArray extends RuntimeSourceImpl1 {
		private RuntimeSource[] arrayElements;
		private String typeName;
		
		RTArray(ArraySource aArray, Component aComponent, RuntimeSource[] elements, String aTypeName) {
			super(aArray, aComponent);

			typeName = aTypeName;
			arrayElements = elements;
		}
					
		/**
		 * @see org.objectweb.kilim.model.RuntimeSource#getValue()
		 */
		public Object getValue() throws KilimException {
			Object eventSrcValue = getEventSourceValue();			
			int size = arrayElements.length;
			Class elementClass = DefaultRuntimeClassLoader.instance.getClass(typeName);
			Object[] resultValue = (Object[]) Array.newInstance(elementClass, size);

			for (int i = 0; i < size; i++) {
				if (arrayElements[i] instanceof RTEventSource) {
					resultValue[i] = eventSrcValue;
				} else {
					RuntimeSource rtS = (RuntimeSource) arrayElements[i];
					//recursive transmission of the event source ...
					rtS.setEventSourceValue(eventSrcValue);
					resultValue[i] = rtS.getValue();
				}
			}
			return resultValue;
		}
		
		/**
		 * @see org.objectweb.kilim.model.RuntimeSource#checkValue()
		 */
		public boolean hasValue()  throws KilimException {
			int size = arrayElements.length;
			for (int i = 0; i < size; i++) {
				if (!arrayElements[i].hasValue()) {
					return false;
				}
			}
			return true;
		}
		
		public boolean checkValue(Stack exclude)  throws KilimException {
			int size = arrayElements.length;
			for (int i = 0; i < size; i++) {
				if (!arrayElements[i].hasValue()) {
					exclude.push(arrayElements[i]);
					boolean isOK = arrayElements[i].checkValue(exclude);
					exclude.pop();
					if (!isOK) {
						return false;
					}
				}
			}
			return true;
		}
	}
	
	/* ************************************************************************************/
	//inner class for a runtime descriptor of a class source
		
	private static class RTClassSource extends RtComponentProvider {
		private Class support;
		private boolean gotAValue;
		
		RTClassSource(ClassSource aSource, Component aComponent) {
			super(aSource, aComponent, null);
		}
			
		String getClassName() {
			return ((ClassSource) getElementDescription()).getClassName();
		}
				
		public Object specificGetValue() throws KilimException {
			if (mappingContext != null) {
				mappingContext.getCallStack().push(this);
			}
			mapper.enterContext(mappingContext);
			
			Object resultValue = mapper.getClassValue(getClassName(), mappingContext);
			gotAValue = true;
			
			if (mappingContext != null) {
				mappingContext.getCallStack().pop();
			}
			mapper.leaveContext(mappingContext);
			return resultValue;
		}
		
		public void addInterfaceListener(RtCollectionPort aInterface) throws KilimException { }
	
		public void removeInterfaceListener(RtCollectionPort aInterface) throws KilimException { }
		
		public void setEventSourceValue(Object aValue) throws KilimException { }
		
		/**
		 * @see org.objectweb.kilim.model.ComponentSource#checkValue()
		 */
		public boolean hasValue() {
			return gotAValue;
		}
		
		public boolean checkValue(Stack exclude) throws KilimException {
			return true;	
		}		
	}
	
	/* ************************************************************************************/
	//inner class for a runtime descriptor of an event source.
		
	private static class RTEventSource extends RtComponentElement implements RuntimeSource {
		private Object target;
		private boolean gotAValue;
	
		RTEventSource() {
			super(null, null);
		}
		
		/**
	 	* @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 	*/
		public String getLocalName() {
			return null;
		}
	
		public void setEventSourceValue(Object aTarget) {
			target = aTarget;
			gotAValue = true;
		}
		
		public boolean isEventSource() {
			return true;
		}
		
		public Object getEventSourceValue() throws KilimException {
			return target;
		}
			
		public Object getValue() throws KilimException {
			return getEventSourceValue();
		}
		
		public void addInterfaceListener(RtCollectionPort aInterface) throws KilimException { }
	
		public void removeInterfaceListener(RtCollectionPort aInterface) throws KilimException { }
		
		public boolean hasValue() {
			return gotAValue;
		}
		
		public boolean checkValue(Stack exclude) throws KilimException {
			return true;
		}		
	}
	
	/* ************************************************************************************/
	//inner class for a runtime descriptor of a null element
		
	private static class RTNullElement extends RtComponentProvider implements RuntimeAction {
			
		RTNullElement(NullElement aNullElement, Component aComponent) {
			super(aNullElement, aComponent, null);
		}
			
		public Object specificGetValue() throws KilimException {
			if (mappingContext != null) {
				mappingContext.getCallStack().push(this);
			}
			mapper.enterContext(mappingContext);
			mapper.getNullElementValue(mappingContext);
			if (mappingContext != null) {
				mappingContext.getCallStack().pop();
			}
			mapper.leaveContext(mappingContext);
			return null;
		}

		public void execute()  throws KilimException {
			mapper.executeNullElement(mappingContext);
		}
		
		public boolean hasValue() {
			return true;
		}
		
		public boolean checkAction(Stack aStack) {
			return true;
		}
		
		public boolean checkValue(Stack exclude) throws KilimException {
			return true;
		}		
	}
	
	/* ************************************************************************************/
	//inner class for a runtime descriptor of a method
	
	private static class RTMethod extends RtComponentProvider implements RuntimeAction {
		private RuntimeSource[] parameters;	
		private String[] typeNames;

		RTMethod(TpMethod aMethod, Component aComponent, RuntimeSource aSupport, RuntimeSource[] params, String[] tpNames) {
			super(aMethod, aComponent, aSupport);
			parameters = params;
			typeNames = tpNames;
		}
				
		boolean isStatic() {
			return ((TpMethod) getElementDescription()).isStatic();
		}
		
		String getMethodName() {
			return ((TpMethod) getElementDescription()).getMethodName();
		}
		
		private Object perform(boolean provides) throws KilimException {
			Object eventSrcValue = getEventSourceValue();
			//specific part of getValue
			RuntimeSource support = (RuntimeSource) getSupport();

			Object resultValue = null;
			if (mappingContext != null) {
				mappingContext.getCallStack().push(this);
			}
			mapper.enterContext(mappingContext);
			
			callStack.push(this);
			
			Object suppObject = null;
			if (support instanceof RTEventSource) {
				suppObject = eventSrcValue;
			} else {
				//recursive transmission of the event source ...
				support.setEventSourceValue(eventSrcValue);
				suppObject = support.getValue();
			}
			
			//this test does not work with a null mapper..
			//if (suppObject == null) {
			//	throw new KilimException("attempt to invoke a method " + getMethodName() + " with a null support " + ((RTMethod) support.getTarget()).getMethodName());
			//}
			
			RtSingleValuePort svp = getCurrentSVP();			
			//If the method is performed for obtaining the value of a single value port, this value may have been obtained
			//as a result of an evaluation cycle...						
			if (provides && svp != null && svp.hasValue()) {
				callStack.pop();
				List iter = svp.getTriggerList(Trigger.BIND);
				if (iter != KILIM.EMPTY_LIST) {
					System.err.println("warning : execution of transformers associated to the port " + svp.getQualifiedName() + " may not follow the tree order : " + svpStack.toString());
				}
				return svp.getBufferedValue();
			}
			
			int paramNumber = parameters.length;
			Object[] paramObjects =  new Object[paramNumber];

			for (int i = 0; i < paramNumber; i++) {
				//If the method is performed for obtaining the value of a single value port, this value may have 
				//been obtained as a result of an evaluation cycle...
				if (provides && svp != null && svp.hasValue()) {
					callStack.pop();
					List iter = svp.getTriggerList(Trigger.BIND);
					if (iter != KILIM.EMPTY_LIST) {
						System.err.println("warning : execution of transformers associated to the port " + svp.getQualifiedName() + " may not follow the tree order : " + svpStack.toString());
					}
					return svp.getBufferedValue();
				}
				paramObjects[i] = null;
				if (parameters[i] != null) {
					if (parameters[i] instanceof RTEventSource) {
						paramObjects[i] = eventSrcValue;
					} else {
						//recursive transmission of the event source ...
						parameters[i].setEventSourceValue(eventSrcValue);
						paramObjects[i] = parameters[i].getValue();
					}
				}
			}

			//call to the mapper that performs the real task ....
			if (provides) {
				if (svp != null && svp.hasValue()) {
					callStack.pop();
					List iter = svp.getTriggerList(Trigger.BIND);
					if (iter != KILIM.EMPTY_LIST) {
						System.err.println("warning : execution of transformers associated to the port " + svp.getQualifiedName() + " may not follow the tree order : " + svpStack.toString());
					}
					return svp.getBufferedValue();
				} else {
					resultValue = mapper.getMethodValue(suppObject, isStatic(), getMethodName(), paramObjects, typeNames, mappingContext);
				}
			} else {
				mapper.executeMethod(suppObject, isStatic(), getMethodName(), paramObjects, typeNames, mappingContext);
			}
			callStack.pop();
			if (mappingContext != null) {
				mappingContext.getCallStack().pop();
			}
			mapper.leaveContext(mappingContext);

			return resultValue;
		}
		
		/**
		 * @see org.objectweb.kilim.model.RtComponentInterface#specificGetValue()
		 */
		public Object specificGetValue() throws KilimException {
			Object resultValue = perform(true);		
			return resultValue;
		}
	
		/**
		 * @see org.objectweb.kilim.model.RuntimeAction#execute()
		 */
		public void execute() throws KilimException {
			perform(false);
		}
		
		/**
		 * @see org.objectweb.kilim.model.ComponentSource#checkValue()
		 */
		public boolean hasValue() throws KilimException {
			if (!getSupport().hasValue()) {
				return false;
			}
			
			int paramNumber = parameters.length;
			for (int i = 0; i < paramNumber; i++) {
				if (!parameters[i].hasValue()) {
					return false;
				}
			}
			return true;
		}
		
		/**
		 * @see org.objectweb.kilim.model.ComponentSource#checkValue()
		 */
		public boolean checkValue(Stack exclude) throws KilimException {
			if (exclude == null) {
				return true;
			}
			
			int eSize = exclude.size();
			
			RuntimeSource lSupport = (RuntimeSource) getSupport().getTarget();
			if (!lSupport.hasValue()) {
				for (int i = 0; i < eSize; i++) {
					if (lSupport == exclude.get(i)) {
						return false;
					}
				}
				
				exclude.push(lSupport);
				boolean isOK = lSupport.checkValue(exclude);
				exclude.pop();
				if (!isOK) {
					return false;
				}
			}
			
			int paramNumber = parameters.length;
			
			for (int i = 0; i < paramNumber; i++) {
				RuntimeSource parm = (RuntimeSource) parameters[i].getTarget();
				if (parm.hasValue()) {
					continue;
				}
				
				for (int j = 0; j < eSize; j++) {
					if (parm == exclude.get(j)) {
						return false;
					}
				}
				
				exclude.push(this);
				boolean isOK = parm.checkValue(exclude);
				exclude.pop();
				
				if (!isOK) {
					return false;
				}
			}
			return true;
		}

		/**
		 * @see org.objectweb.kilim.model.RuntimeAction#checkAction()
		 */
		public boolean checkAction(Stack aStack) throws KilimException {
			RuntimeSource lSupport = (RuntimeSource) getSupport().getTarget();
			if (aStack == null) {
				return true;
			}
			
			if (!lSupport.checkValue(aStack)) {
				return false;
			}
			
			int stackSize = aStack.size();
			int paramNumber = parameters.length;
			for (int i = 0; i < paramNumber; i++) {
				RuntimeSource parm = (RuntimeSource) parameters[i].getTarget();
				if (!parm.checkValue(aStack)) {
					return false;
				}
			}
			return true;
		}
	}
	
	/* ************************************************************************************/
	//inner class for a runtime descriptor of a constructor
		
	private static class RTConstructor extends RtComponentProvider implements RuntimeAction {
		private RuntimeSource[] parameters;
		private String[] typeNames;

		RTConstructor(TpConstructor aCtor, Component aComponent, RuntimeSource aSupport, RuntimeSource[] params, String[] tNames) {
			super(aCtor, aComponent, aSupport);
			parameters = params;
			typeNames = tNames;
		}
				
		public Object specificGetValue() throws KilimException {
			Object resultValue = perform(true);
			return resultValue;
		}
			
		private Object perform(boolean provides) throws KilimException {
			if (mappingContext != null) {
				mappingContext.getCallStack().push(this);
			}
			mapper.enterContext(mappingContext);
						
			callStack.push(this);
						
			Object eventSrcValue = getEventSourceValue();
			RuntimeSource support = (RuntimeSource) getSupport();
			if (support == null) { 
				throw  new KilimException("attempt to get the value of a non initialized constructor");
			}

			Class clazz = null;
			if (support.isEventSource()) {
				clazz = (Class) eventSrcValue;
			} else {
				clazz = (Class) support.getValue();
			}

			//If the method is performed for obtaining the value of a single value port, this value may have been obtained
			//as a result of an evaluation cycle...
			RtSingleValuePort svp = getCurrentSVP();

			if (provides && svp != null && svp.hasValue()) {
				callStack.pop();
				List iter = svp.getTriggerList(Trigger.BIND);
				if (iter != KILIM.EMPTY_LIST) {
					System.err.println("warning : execution of transformers associated to the port " + svp.getQualifiedName() + " may not follow the tree order : " + svpStack.toString());
				}
				return svp.getBufferedValue();
			}
				
			int paramNumber = parameters.length;
			Object[] paramObjects =  new Object[paramNumber];

			for (int i = 0; i < paramNumber; i++) {
				//If the method is performed for obtaining the value of a single value port, this value may have 
				//been obtained as a result of an evaluation cycle...

				if (provides && svp != null && svp.hasValue()) {
					callStack.pop();
					List iter = svp.getTriggerList(Trigger.BIND);
					if (iter != KILIM.EMPTY_LIST) {
						System.err.println("warning : execution of transformers associated to the port " + svp.getQualifiedName() + " may not follow the tree order : " + svpStack.toString());
					}
					return svp.getBufferedValue();
				}

				paramObjects[i] = null;
				if (parameters[i] != null) {
					if (parameters[i] instanceof RTEventSource) {
						paramObjects[i] = eventSrcValue;
					} else {
						//recursive transmission of the event source ...
						parameters[i].setEventSourceValue(eventSrcValue);
						paramObjects[i] = parameters[i].getValue();
					}
				}
			}
			Object resultValue = null;
			
			if (provides) {
				if (svp != null && svp.hasValue()) {
					callStack.pop();
					List iter = svp.getTriggerList(Trigger.BIND);
					if (iter != KILIM.EMPTY_LIST) {
						System.err.println("warning : execution of transformers associated to the port " + svp.getQualifiedName() + " may not follow the tree order : " + svpStack.toString());
					}
					return svp.getBufferedValue();
				} else {
					resultValue = mapper.getConstructorValue(clazz, paramObjects, typeNames, mappingContext);
				}
			} else {
				mapper.getConstructorValue(clazz, paramObjects, typeNames, mappingContext);
			}
			
			callStack.pop();
			if (mappingContext != null) {
				mappingContext.getCallStack().pop();
			}	
			mapper.leaveContext(mappingContext);	

			return resultValue;
		}
	
		/**
		 * @see org.objectweb.kilim.model.RuntimeAction#execute()
		 */
		public void execute() throws KilimException {
			perform(false);
		}
		
		/**
		 * @see org.objectweb.kilim.model.ComponentSource#checkValue()
		 */
		public boolean hasValue() throws KilimException {
			if (!getSupport().hasValue()) {
				return false;
			}
			
			int paramNumber = parameters.length;
			for (int i = 0; i < paramNumber; i++) {
				if (!parameters[i].hasValue()) {
					return false;
				}
			}
			return true;
		}
		
				/**
		 * @see org.objectweb.kilim.model.ComponentSource#checkValue()
		 */
		public boolean checkValue(Stack exclude) throws KilimException {
			if (exclude == null) {
				return true;
			}
			
			int eSize = exclude.size();
			
			RuntimeSource lSupport = (RuntimeSource) getSupport().getTarget();
			if (!lSupport.hasValue()) {
				for (int i = 0; i < eSize; i++) {
					if (lSupport == exclude.get(i)) {
						return false;
					}
				}
				
				exclude.push(lSupport);
				boolean isOK = lSupport.checkValue(exclude);
				exclude.pop();
				if (!isOK) {
					return false;
				}
			}
			
			int paramNumber = parameters.length;
			
			for (int i = 0; i < paramNumber; i++) {
				RuntimeSource parm = (RuntimeSource) parameters[i].getTarget();
				if (parm.hasValue()) {
					continue;
				}
				
				for (int j = 0; j < eSize; j++) {
					if (parm == exclude.get(j)) {
						return false;
					}
				}
				
				exclude.push(this);
				boolean isOK = parm.checkValue(exclude);
				exclude.pop();
				
				if (!isOK) {
					return false;
				}
			}
			return true;
		}

		/**
		 * @see org.objectweb.kilim.model.RuntimeAction#checkAction()
		 */
		public boolean checkAction(Stack aStack) throws KilimException {
			RuntimeSource lSupport = (RuntimeSource) getSupport().getTarget();
			
			if (aStack == null) {
				return true;
			}
			
			if (!lSupport.checkValue(aStack)) {
				return false;
			}
			
			int stackSize = aStack.size();
			int paramNumber = parameters.length;
			for (int i = 0; i < paramNumber; i++) {
				RuntimeSource parm = (RuntimeSource) parameters[i].getTarget();
				if (!parm.checkValue(aStack)) {
					return false;
				}
			}
			return true;
		}
	}
	
	/* ************************************************************************************/
	
	private static class RTReference extends RuntimeSourceImpl1 implements RuntimeAction {
		private RuntimeElement target;
	
		RTReference(TemplateElementImpl aElement, Component aComponent) {
			super(aElement, aComponent);
		}


		String getTargetName() {
			return ((Reference) getElementDescription()).getTargetName();
		}
	
		public RuntimeElement getTarget() {
			if (target == null) {
				return null;
			}
			if (target instanceof RTReference) {
				return ((RTReference) target).getTarget();
			}
			return (RuntimeElement) target;
		}
		
		void setTarget(RuntimeElement aTarget) {
			target = aTarget;
		}
					
		public void execute() throws KilimException {
			if (target == null) {
				throw  new KilimException("attempt to execute an action from an unbound reference " + this + " in component ");
			}
		
			RuntimeElement tgt = target;
			if (tgt instanceof RTReference) {
				tgt = ((RTReference) tgt).getTarget();
			}
			((RuntimeAction) tgt).execute();

		}
		
		public Object getValue() throws KilimException {
			if (target == null) {
				throw  new KilimException("attempt to get a value from an unbound reference " + this + " in component ");
			}
		
			RuntimeElement tgt = target;
			if (tgt instanceof RTReference) {
				tgt = ((RTReference) tgt).getTarget();
			}
			return ((RuntimeSource) tgt).getValue();
		}
		
		/**
		 * @see org.objectweb.kilim.model.RuntimeSource#checkValue()
		 */
		public boolean hasValue() throws KilimException {
			if (target == null) {
				throw  new KilimException("attempt to check the value from an unbound reference " + this + " in component ");
			}
			
			if (target instanceof RuntimeSource) {
					return ((RuntimeSource) target).hasValue();
			} 
			throw  new KilimException("attempt to check the value from non source element " + this + " in component ");			
		}
		
		/**
		 * @see org.objectweb.kilim.model.RuntimeAction#checkAction()
		 */
		public boolean checkAction(Stack aStack) throws KilimException {
			if (target == null) {
				throw  new KilimException("attempt to check the value from an unbound reference " + this + " in component ");
			}
			
			if (target instanceof RuntimeAction) {
					return ((RuntimeAction) target).checkAction(aStack);
			}
			throw  new KilimException("attempt to check the action from non action element " + this + " in component ");
		}
		
		/**
		 * @see org.objectweb.kilim.model.RuntimeAction#checkAction()
		 */
		public boolean checkValue(Stack aStack) throws KilimException {
			if (target == null) {
				throw  new KilimException("attempt to check the value from an unbound reference " + this + " in component ");
			}
			
			if (target instanceof RuntimeSource) {
					return ((RuntimeSource) target).checkValue(aStack);
			}
			throw  new KilimException("attempt to check the action from non action element " + this + " in component ");
		}				
	}
	
	/**
	 * A factory method for creating a new component.
	 * @param template : the template to be used
	 * @return Component
	 * @throws KilimException : generated if template is null.
	 */
	public static Component newComponent(TemplateDescription template) throws KilimException {
		ComponentFactory factory =  new ComponentFactory("", template, null, null);
		factory.initializeInstanciation();
		factory.initNewComponent();
		factory.resolveReferences();
		factory.installBindings();
		factory.installTriggers();
		factory.installPlugs();
		factory.finalizeInstanciation();
		return factory.getComponent();
	}
		
	/**
	 * A factory method for creating a new component.
	 * @param template : the template to be used.
	 * @param aStrgy : the implementation strategy to be used.
	 * @return Component
	 * @throws KilimException : generated if template is null.
	 */
	public static Component newComponent(TemplateDescription template, InstanciationStrategy aStrgy) throws KilimException {
		if (aStrgy == null) {
			throw new KilimException("attempt to assign a null instanciation strategy for template " + template.getName());
		}
		KilimConfiguration.setInstanciationStrategy(aStrgy);
		return newComponent(template);
	}
	
	private ComponentFactory(String aName, TemplateDescription aTemplate, ComponentFactory aFactory, RtComponent aComponent) throws KilimException {
		super(aName, aFactory, null);
		if (aTemplate == null) {
			throw  new KilimException("attempt to construct a component without template ");
		}
		localName = aName;
		containingFactory = aFactory;
		if (containingFactory != null) {
			containingFactory.addChildNamingContext(aName, this);
		}
		
		containingComponent = aComponent;
		currentTemplate = aTemplate;
		//instanciationStrategy = KilimConfiguration.getInstanciationStgy();
		setExternalReferences(KilimConfiguration.getExternalReferences());
	}
		
	/**
	 * returns the local name of the factory (which is similar to the component name !!!!)
	 * @return String
	 */
	public String getLocalName() {
		if (localName == null) {
			return "";
		}
		return localName;
	}

	/**
	 * returns the component built by the factory.
	 * This method returns a reference of the Component meta-object bound to the "this" interface.
	 * @return Component
	 */
	public Component getComponent() {
		return currentComponent;
	}
	
	/**
	 * In the present implementation factories are used as naming contexts. As a consequence they are organized according to a tree-like structure.
	 * This method therefore returns the containing factory.
	 * @return ComponentFactory
	 */
	public ComponentFactory getContainingFactory() {
		return containingFactory;
	}

		
	/**
	 * In the present implementation factories are used as naming contexts. As a consequence they are organized according to a tree-like structure.
	 * This method adds a SubFactory.
	 * @param aElement : the Child factory to be added.
	 * @throws KilimException :
	 */
	public void addSubFactory(ComponentFactory aElement) throws KilimException {
		if (aElement == null) {
			throw  new KilimException("attempt to add a null subfactory to factory " + getQualifiedName());
		}
				
		String lName = aElement.getLocalName();
		
		if (subFactories == null) {
			subFactories = new ArrayList();
		}
		
		if (containsElement(subFactories, lName)) {
			throw  new KilimException("name clash when adding the component " + lName + "in factory " + getQualifiedName());
		}
		
		subFactories.add(aElement);
	}
	
	/**
	 * In the present implementation factories are used as naming contexts. As a consequence they are organized according to a tree-like structure.
	 * This method removes a Sub Factory.
	 * @param aElement :
	 * @throws KilimException :
	 */
	public void removeSubFactory(ComponentFactory aElement) throws KilimException {
		if (aElement == null) {
			throw  new KilimException("attempt to remove a null factory in factory " + getQualifiedName());
		}
		
		if (subFactories == null) {
			throw  new KilimException("no subfactory in current factory");
		}
		
		Object previous = removeElement(subFactories, aElement.getLocalName());
		if (previous == null) {	
			throw  new KilimException("subfactory " + aElement.getLocalName() + " unknown in current factory " + getQualifiedName());
		}		
	}
	
	/**
	 *In the present implementation factories are used as naming contexts. As a consequence they are organized according to a tree-like structure.
	 * This method returns a SubFactory.
	 * @param aName :
	 * @return ComponentFactoryElement
	 */
	public ComponentFactory getSubFactory(String aName) {
		if (subFactories == null) {
			return null;
		}
		return (ComponentFactory) getElement(subFactories, aName);
	}
	
	/**
	 * Method getComponentFactorys.
	 * @return Iterator
	 */
	public Iterator getSubFactories() {
		if (subFactories == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return subFactories.iterator();
	}
			
	/**
	 * Method newComponent.
	 * @param instance :
	 * @return Component :
	 * @throws KilimException :
	 */
	private Component newComponent(Instance instance) throws KilimException {
		TemplateDescription template = instance.getTemplate();
		ComponentFactory factory = new ComponentFactory(instance.getLocalName(), template, this, (RtComponent) getComponent());
		addSubFactory(factory);
		factory.initNewComponent();
		return factory.getComponent();
	}		
	
	private void initNewComponent() throws KilimException {
		//creation of the new component meta object
		Instance instance =  new Instance(localName, KILIM.PUBLIC, currentTemplate, null);
		currentComponent =  new RtComponent(instance, containingComponent, this);
		//creation of the "THIS" interface referencing the meta object itself
		//RtSingleValuePort thisInterface = new RtSingleValuePort(Port.THIS_PORT, currentComponent, fMapper, fMappingContext);
		RtSingleValuePort thisInterface = new RtSingleValuePort(Port.THIS_PORT, currentComponent);
		currentComponent.addInterface(thisInterface);
		addBoundName("THIS", thisInterface);
		thisInterface.bindProvider(currentComponent, false);
		//setInstanciationMger(currentComponent);

		//instanciation of all local provider references
		TemplateDescription template1 = currentTemplate;
		
		Iterator iter = null;
		do {
			iter = template1.getProviderReferences(true);
			while (iter.hasNext()) {
				Reference elem = (Reference) iter.next();
				RTReference ref = newInnerReference(elem);
				addReference(elem.getTargetName(), ref);
			}
			template1 = template1.getSuperTemplate();
		} while (template1 != null);
		
		//instanciation of all local transformer references
		template1 = currentTemplate;
		do {
			iter = template1.getTransformerReferences(true);
			while (iter.hasNext()) {
				Reference elem = (Reference) iter.next();
				RTReference ref = newInnerReference(elem);
				addReference(elem.getTargetName(), ref);
			}
			template1 = template1.getSuperTemplate();
		} while (template1 != null);

		ComponentElement result = null;

		//instanciation of all local ports not declared as slot members
		iter = currentTemplate.getAllInterfaces(false);
		while (iter.hasNext()) {
			BasicNamedElement elem = (BasicNamedElement) iter.next();
			result = newComponentElement1(elem);
			addBoundName(elem.getLocalName(), result);
		}
				
		//instanciation of all local transformers
		iter = currentTemplate.getTransformers(false);
		while (iter.hasNext()) {
			BasicNamedElement elem = (BasicNamedElement) iter.next();
			result = newComponentElement1(elem);
			addBoundName(elem.getLocalName(), result);
		}

		//instanciation of all local ports declared as slot members
		iter = currentTemplate.getSlots(false);
		while (iter.hasNext()) {
			Slot elem = (Slot) iter.next();
			String sName = elem.getLocalName();
			//SlotFactory sFactory = new SlotFactory(sName, this, fMapper, fMappingContext);
			SlotFactory sFactory = new SlotFactory(sName, this);
			RtComponentSlot rSlot = sFactory.newSlot(elem);
			((RtComponent) getComponent()).addSlot(rSlot);
		}		
				
		//instanciation of all local sinstances
		iter = currentTemplate.getInstances(false);
		while (iter.hasNext()) {
			Instance elem = (Instance) iter.next();
			Component compo1 = newComponent(elem);
			addBoundName(elem.getLocalName(), compo1);
		}			
	}
	
	private ComponentElement newComponentElement1(BasicElement elem) throws KilimException {
		ComponentElement result = null;
		
		switch(elem.getKind()) {
			case KILIM.PORT :
				result = newInterface((Port) elem);
				currentComponent.addInterface((RtComponentInterface) result);
				return result;
			case KILIM.ARRAY :
				return newInnerArraySource((ArraySource) elem);			
			case KILIM.CLASS :
				return newInnerClassSource((ClassSource) elem);
			case KILIM.GETTER :
				return newInnerGetter((TpGetter) elem);
			case KILIM.SETTER :
				return newInnerSetter((TpSetter) elem);
			case KILIM.METHOD :
				return newInnerMethod((TpMethod) elem);
			case KILIM.CONSTRUCTOR :
				return newInnerConstructor((TpConstructor) elem);
			case KILIM.REFERENCE :
				return getReference(((Reference) elem).getTargetName());
			case KILIM.PROVIDER :
				result = newComponentElement1(((Provider) elem).getSource());
				currentComponent.addInterface((RtComponentInterface) result);
				return result;
			case KILIM.TRANSFORMER :
				result = newComponentElement1(((Transformer) elem).getAction());
				return result;
			case KILIM.PROPERTY :
				result = newProperty((Property) elem);
				currentComponent.addInterface((RtComponentInterface) result);
				return result;
			case KILIM.EVENT_SOURCE :
				return newInnerEventSource((EventSource) elem);
			case KILIM.NULL_ELEMENT :
					//return new RTNullElement((NullElement) elem, getComponent(), fMapper, fMappingContext);
					return new RTNullElement((NullElement) elem, getComponent());
			default :
				throw  new KilimException("attempt to create an unknown kind (" + elem.getClass() + ") of inner element in component " + getQualifiedName());
		}
	}

	private RtComponentProperty newProperty(Property aProperty) throws KilimException {
		return new RtComponentProperty(aProperty, getComponent());
	}
	
	private RtComponentInterface newInterface(Port port) throws KilimException {
		RtComponentInterface interf = null;
		if (port.isUnary()) {
			interf = new RtSingleValuePort(port, getComponent());
		} else {
			interf = new RtCollectionPort(port, getComponent());
		}
		return interf;
	}

	private RTArray newInnerArraySource(ArraySource aArray) throws KilimException {
		int size = aArray.getCurrentSize();
		RuntimeSource[] values = new RuntimeSource[size];
		for (int i = 0; i < size; i++) {
			 values[i] = (RuntimeSource) newComponentElement1(aArray.getElement(i));
		}
		
		return new RTArray(aArray, getComponent(), values, aArray.getTypeName());
	}
	
	private RTClassSource newInnerClassSource(ClassSource aSource) throws KilimException {
		String className = aSource.getClassName();
		return  new RTClassSource(aSource, getComponent());
	}

	private RTGetter newInnerGetter(TpGetter aGetter) throws KilimException {
		RuntimeSource support = (RuntimeSource) newComponentElement1(aGetter.getSupport());
		return  new RTGetter(aGetter, getComponent(), support);
	}

	private RTSetter newInnerSetter(TpSetter aSetter) throws KilimException {
		RuntimeSource aSupport = (RuntimeSource) newComponentElement1(aSetter.getSupport());
		RuntimeSource aValue = (RuntimeSource) newComponentElement1(aSetter.getValue());
		RTSetter result = new RTSetter(aSetter, getComponent(), aSupport, aValue);
		return result;
	}
	
	private RTMethod newInnerMethod(TpMethod aMethod) throws KilimException {
		RuntimeSource aSupport = (RuntimeSource) newComponentElement1(aMethod.getSupport());
		int parmNumber = aMethod.getParameterNumber();
		RuntimeSource[] params = new RuntimeSource[parmNumber];
		String[] typeNames = new String[parmNumber];
		
		for (int i = 0; i < parmNumber ; i++) {
			params[i] = (RuntimeSource) newComponentElement1(aMethod.getParameter(i).getTarget());
			typeNames[i] = aMethod.getParameter(i).getTypeName();
		}		
		return new RTMethod(aMethod, getComponent(), aSupport, params, typeNames);
	}	
	
	private RTConstructor newInnerConstructor(TpConstructor aCtor) throws KilimException {
		RuntimeSource aSupport = (RuntimeSource) newComponentElement1(aCtor.getSupport());		
		int parmNumber = aCtor.getParameterNumber();
		RuntimeSource[] params = new RuntimeSource[parmNumber];
		String[] typeNames = new String[parmNumber];
		
		for (int i = 0; i < parmNumber ; i++) {
			params[i] = (RuntimeSource) newComponentElement1(aCtor.getParameter(i).getTarget());
			typeNames[i] = aCtor.getParameter(i).getTypeName();
		}
		return  new RTConstructor(aCtor, getComponent(), aSupport, params, typeNames);
	}	
	
	private RTReference newInnerReference(Reference aReference) throws KilimException {
		if (references == null) {
			references = new LinkedHashMap();
		}
		RTReference ref = new RTReference(aReference, getComponent());
		return ref;
	}
	
	private RTEventSource newInnerEventSource(EventSource aSource) throws KilimException {
		return new RTEventSource();
	}
	
	/**
	 * returns a new component which is a copy of aComponent (in the state defined by the template). All operations performed at runtime is ignored.
	 * @param aComponent : the component to be forked
	 * @return Component
	 * @throws KilimException :
	 */
	public Component fork(Component aComponent) throws KilimException {
		if (aComponent == null) {
			throw new KilimException("attempt to fork a null component in component factory " + getQualifiedName());
		}
		
		Instance instance = (Instance) aComponent.getElementDescription().clone();
		instance.setLocalName("%fork" + KILIM.getUniqueIndex());
		
		Component nCompo = newComponent(instance);
		addBoundName(nCompo.getLocalName(), nCompo);
		ComponentFactory factory = (ComponentFactory) nCompo.getFactory();
		factory.resolveReferences();
		factory.installBindings();
		factory.installTriggers();
		factory.installPlugs();
		Iterator iter = nCompo.getInterfaces();
		while (iter.hasNext()) {
			RtComponentInterface interf = (RtComponentInterface) iter.next();
			Iterator iter1 = interf.getInterfaceListeners();
			while (iter1.hasNext()) {
				RtCollectionPort interf1 = (RtCollectionPort) iter1.next();
				interf1.bindProvider(interf, false);
			}
		}
		
		Iterator iter1 = aComponent.getPlugTos();
		while (iter1.hasNext()) {
			ComponentSlot rtSlot = (ComponentSlot)  iter1.next();
			rtSlot.plug(nCompo);
		}
		factory.finalizeInstanciation();
		return nCompo;
	}
		
	/**
	 * Method addReference.
	 * @param aName :
	 * @throws KilimException :
	 */
	private void addReference(String aName, RTReference aReference) throws KilimException {		
		if (aReference == null) {
			throw  new KilimException("attempt to add a null target in component " + getQualifiedName());
		}
		
		if (aName == null) {
			throw  new KilimException("attempt to add a reference with a null target name in component " + getQualifiedName());
		}
			
		if (references == null) {
			references = new LinkedHashMap();
		}

		if (!references.containsKey(aName)) {
			references.put(aName, aReference);
		}
	}
	
	/**
	 * Method removeSubContext.
	 * @param aName :
	 * @throws KilimException :
	 */
	private void removeReference(String aName) throws KilimException {
		if (aName == null) {
			throw  new KilimException("attempt to remove a reference through a null name in component " + getQualifiedName());
		}
		
		if (references  == null) {
			throw  new KilimException("no unbound names in current context");
		}
		
		Object previous = references.remove(aName);
		if (previous == null) {	
			throw  new KilimException("name " + aName + " unknown in current context ");
		}		
	}
	
	/**
	 * Method containsReference.
	 * @param aName :
	 * @return boolean
	 * @throws KilimException :
	 */
	private RTReference getReference(String aName) throws KilimException {
		if (aName == null) {
			throw  new KilimException("null name in containsReference of Component ");
		}
		if (references == null) {
		//	throw  new KilimException("attempt to get name " + aName + " in a component without references " + getLocalName());
		return null;
		}
		return (RTReference) references.get(aName);
	}
		
	/**
	 * Method resolveReference.
	 * This method recursively executes the resolution process in all factories.
	 * @throws KilimException :
	 */
	private void resolveReferences() throws KilimException {
		if (subFactories != null) {
			Iterator iter = subFactories.iterator();
			while (iter.hasNext()) {
				ComponentFactory factory = (ComponentFactory) iter.next();
				factory.resolveReferences();
			}
		}

		if (references != null) {
			Iterator iter = references.values().iterator();
			while (iter.hasNext()) {
				RTReference ref = (RTReference) iter.next();
				ComponentElement elem = resolveReference(ref.getTargetName(), getComponent());
				ref.setTarget((RuntimeElement) elem);
			}
		}
	}
				
	/**
	 * Method getInstances.
	 * @return Iterator
	 */
	private Iterator getReferences() {
		if (references == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return references.values().iterator();
	}
	
	/**
	 * Method installBinding.
	 * @param aBinding :
	 * @throws KilimException :
	 */
	private void installBinding(Binding aBinding) throws KilimException {
		String portName = aBinding.getPortName();
		RtComponentInterface interf = (RtComponentInterface) resolveReference(portName, getComponent());
		
		if (interf == null) {
			String compoQName = getComponent().getQualifiedName();
			throw  new KilimException ("attempt to install a binding on an unknown port " + portName + " in component " + compoQName + "(template : " + getFullTemplateSequence() + ")");
		}
		
		Iterator iter = aBinding.getBoundProviders();
		while (iter.hasNext()) {
			BasicElement elem1 = (BasicElement) iter.next();
			ComponentElement compo = null;
			if (elem1 instanceof Reference) {
				compo = resolveReference(((Reference) elem1).getTargetName(), getComponent());
				if (compo == null) {
					String compoQName = getComponent().getQualifiedName();
					throw  new KilimException("attempt to bind an unknown reference " + ((Reference) elem1).getTargetName() + " to port " + portName + " in component " + compoQName + "(template : " + getFullTemplateSequence() + ")");
				}
				
				interf.bindProvider((RuntimeSource) compo, false);
			} else {
				compo = newComponentElement1(elem1);
				if (compo == null) {
					String compoQName = getComponent().getQualifiedName();
					throw  new KilimException("attempt to bind a null provider to port " + portName + " in component " + compoQName + "(template : " + getFullTemplateSequence() + ")");
				} else {
					interf.bindProvider((RuntimeSource) compo, false);
				}
			}
		}
	}
	
	private String getFullTemplateSequence() {
		TemplateDescription tmp0 = getComponent().getTemplate();
		String tmpName = tmp0.getName();
		while ((tmp0 = tmp0.getSuperTemplate()) != null) {
			tmpName = tmpName + ", " + tmp0.getName();
		}
		return tmpName;
	}
		
	/**
	 * Method installBindings.
	 * This method recursively installs all bindings.
	 * @throws KilimException :
	 */
	public void installBindings() throws KilimException {
		Component compo = getComponent();
		TemplateDescription template = compo.getTemplate();	
		Iterator iter = template.getBindings(false);
		while (iter.hasNext()) {
			installBinding((Binding) iter.next());
		}
			
		iter = getSubFactories();
		while (iter.hasNext()) {
			((ComponentFactory) iter.next()).installBindings();
		}	
	}
	
		private void installPlug(Plug aPlug) throws KilimException {
		if (aPlug == null) {
			throw  new KilimException ("attempt to install a plug binding through a null plug in component " + getComponent().getQualifiedName());
		}
		
		String slotName = aPlug.getSlotName();
		RtComponentSlot rSlot = (RtComponentSlot) getComponent().getSlot(slotName);
		if (rSlot == null) {
			throw  new KilimException ("attempt to install a plug binding through a null slot in component " + getComponent().getQualifiedName());
		}
		
		String instanceName = aPlug.getInstanceName();
		Component compo = (Component) resolveReference(instanceName, getComponent());
		rSlot.plug(compo);
	}

	/**
	 * Method installPlugs.
	 * This method recursively installs all bindings.
	 * @throws KilimException :
	 */
	public void installPlugs() throws KilimException {
		Component compo = getComponent();
		TemplateDescription template = compo.getTemplate();	
			
		Iterator iter = getSubFactories();
		while (iter.hasNext()) {
			((ComponentFactory) iter.next()).installPlugs();
		}
		
		iter = template.getPlugs(false);
		while (iter.hasNext()) {
			installPlug((Plug) iter.next());
		}		
	}
	
	/**
	 * Method installTrigger.
	 * @param aTrigger
	 * @throws KilimException
	 */
	private void installTrigger(Trigger aTrigger) throws KilimException {
		String sourceName = aTrigger.getSourceName();
		ComponentElement elem = (ComponentElement) resolveReference(sourceName, getComponent());
		if (elem == null) {
			throw  new KilimException ("attempt to install a binding on an unknown port " + sourceName + " in component " + getComponent().getQualifiedName());
		}
		
		RuntimeTrigger rContext = new RuntimeTrigger(aTrigger.getEventKind(), ((RtComponentInterface) elem));
		Iterator iter = aTrigger.getTransformers();
		while (iter.hasNext()) {
			BasicElement elem1 = (BasicElement) iter.next();
			ComponentElement elem2 = null;
			if (elem1 instanceof Reference) {
				elem2 = resolveReference(((Reference) elem1).getTargetName(), getComponent());
				if (elem2 == null) {
					throw  new KilimException("attempt to bind an unknown reference " + ((Reference) elem1).getTargetName() + " to port "
					+ sourceName + " in component " + getComponent().getQualifiedName());
				}
				rContext.addTransformer((RuntimeAction) elem2);
			} else {
				elem2 = newComponentElement1(elem1);
				if (elem2 == null) {
					throw  new KilimException("attempt to bind a null provider to port " + sourceName + " in component " + getComponent().getQualifiedName());
				} else {
					rContext.addTransformer((RuntimeAction) elem2);
				}
			}
		}
		((RtComponentInterface) elem).addTrigger(rContext);
	}
	
	/**
	 * Method installTriggers.
	 * This method recursively installs all triggers
	 * @throws KilimException :
	 */
	public void installTriggers() throws KilimException {
		Component compo = getComponent();
		TemplateDescription template = compo.getTemplate();
		Iterator iter = template.getTriggers(false);
		while (iter.hasNext()) {
			Trigger trig = (Trigger) iter.next();
			installTrigger(trig);
		}
		
		iter = getSubFactories();
		while (iter.hasNext()) {
			((ComponentFactory) iter.next()).installTriggers();
		}		
	}
		
	/**
	 * Method finalizeInstanciation.
	 * This method is executed by the component factory after any other resolution/instanciation actions.
	 * This method recursively executes in each factory the finalizeInstanciation() method provided by the 
	 * corresponding InstanciationMger.It is used among other things to associate an instanciation manager
	 * to each component.
	 * @throws KilimException :
	 */
	public void finalizeInstanciation() throws KilimException {
		//recursive invocation of finalizeInstnaciation on subfactories (and thus subcomponents).
		Iterator iter = getSubFactories();
		while (iter.hasNext()) {
			((ComponentFactory) iter.next()).finalizeInstanciation();
		}
		//is empty for the moment : no action required ....
	}
	
	/**
	 * Method initializeInstanciation
	 * This method is executed once by the component factory before any other actions. It can be used to settle 
	 * resources required by the compoonent factories or any preliminary tasks.
	 * @throws KilimException :
	 */
	public void initializeInstanciation() throws KilimException {
		//is empty for the moment : no action required ....
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	protected boolean containsElement(List aList, String aName) throws InternalException {
		int lSize = aList.size();
		if (lSize == 0) {
			return false;
		}
		
		for (int i = 0; i < lSize; i++) {
			ComponentFactory el = (ComponentFactory) aList.get(i);
			if (aName.equals(el.getLocalName())) {
				return true;
			}
		}
		return false;
	}
	
	protected Object removeElement(List aList, String aName) throws InternalException {
		int lSize = aList.size();
		if (lSize == 0) {
			return null;
		}
		
		for (int i = 0; i < lSize; i++) {
			ComponentFactory el = (ComponentFactory) aList.get(i);
			if (aName.equals(el.getLocalName())) {
				return aList.remove(i);
			}
		}
		return null;
	}
	
	protected Object getElement(List aList, String aName) throws InternalException {
		int lSize = aList.size();
		if (lSize == 0) {
			return null;
		}
		
		for (int i = 0; i < lSize; i++) {
			ComponentFactory el = (ComponentFactory) aList.get(i);
			if (aName.equals(el.getLocalName())) {
				return el;
			}
		}
		return null;
	}
}

/**
 * This is a base for all classes that implement runtime sources that do not support
 * event sources and inputs (i.e. array, class source, null source, reference).
 *  @author horn
*/

abstract class RuntimeSourceImpl1 extends RtComponentSource {
	
	public RuntimeSourceImpl1(TemplateElementImpl aElement, Component aComponent) {
		super(aElement, aComponent);
	}

	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 */
	public String getLocalName() {
		return null;
	}
	
	public void addInterfaceListener(RtCollectionPort aInterface) throws KilimException { }
	
	public void removeInterfaceListener(RtCollectionPort aInterface) throws KilimException { }
	
	public boolean isEventSource() {
		return false;
	}
	
	public Object getEventSourceValue() throws KilimException {
		return null;
		//throw new KilimException("attempt to get the event source of element not supporting event sources ");
	}
	
	public void setEventSourceValue(Object aSource) throws KilimException {
		//throw new KilimException("attempt to set the event source to a element not supporting event sources ");
	}
}