/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;
import java.util.LinkedHashMap;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.NamedElement;
import org.objectweb.kilim.description.Port;
import org.objectweb.kilim.description.Trigger;
	
/**
 * @author horn
 * RtCollectionPort is the class for collection ports.
 */
	
public class RtCollectionPort extends RtComponentInterface {
	private static final Integer FALSE = new Integer(0);
	private static final Integer BIND_TRUE = new Integer(1);
	private static final Integer UNBIND_TRUE = new Integer(2);
		
	private LinkedHashMap providers;
	
	/**
	 * The public constructor of rtCollectionPort.
	 * @param aPort : the corresponding template element.
	 * @param aContainer : the container in which the port is declared and defined (is either a RtComponentSlot or a rtComponent.
	 */
	public RtCollectionPort(Port aPort, ContainerElement aContainer) {
		super(aPort, aContainer);
	}

	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isSingleValueInterface()
	 */
	public boolean isSingleValuePort() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isCollectionInterface()
	 */
	public boolean isCollectionPort() {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isProvider()
	 */
	public boolean isProvider() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isProperty()
	 */
	public boolean isProperty() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentSource#hasValue()
	 */
	public boolean hasValue() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#checkValue(Stack)
	 */
	public boolean checkValue(Stack exclude) throws KilimException {
		return false;
	}

	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 */
	public String getLocalName() {
		NamedElement elem = (NamedElement) getElementDescription();
		return elem.getLocalName();
	}
	
	/**
	 * Triggers associated to collection ports must be performed for each bound source when the getValue() or unbound method is invoked on the source.
	 * This method stores a flag indicating that triggers have been executed on a given source.
	 * @param aProvider : the value source.
	 * @param aEKind : the kind of event. The 2 possible values are Trigger.BIND (when getValue is effectively performed) or Trigger.UNBIND
	 * (when unbound is executed).
	 * @param isU : is true for update.
	 * @throws KilimException : generated if aProvider is null, if no sources are bound to the port, if the source is unknown.
	 */
	public void setTriggersDone(RuntimeSource aProvider, int aEKind, boolean isU) throws KilimException  {
		if (aProvider == null) {
			throw new KilimException("attempt to set the use state of a null provider to an interface " + getQualifiedName());
		}
					
		if (providers == null) {
			throw new KilimException("attempt to set the use state of a provider in an empty nary port " + getQualifiedName());
		}
		
		boolean cont = providers.containsKey(aProvider);

		if (!cont) {
			throw new InternalException("attempt to get the inner state of an unknown provider in an nary port " + getQualifiedName() + " # " + aProvider);
		} else {							
			switch(aEKind) {
				case Trigger.BIND :
					providers.put(aProvider, isU ? BIND_TRUE : FALSE);
					break;
				case Trigger.UNBIND :
					providers.put(aProvider, isU ? UNBIND_TRUE : FALSE);
					break;
				default :
					throw new InternalException("attempt to get the inner state of an unknown provider in an nary port " + getQualifiedName());
			}
		}
	}
	
	/**
	 * returns whether the triggers associated to a collection port have been executed for a given source.
	 * @param aProvider : the value source.
	 * @param aEKind : the kind of event. The 2 possible values are Trigger.BIND (when getValue is effectively performed) or Trigger.UNBIND
	 * (when unbound is executed).
	 * @return boolean 
	 * @throws KilimException : generated if aProvider is null, if the collection port is empty or if the port is not bound to the source.
	 */
	public boolean triggersDone(RuntimeSource aProvider, int aEKind) throws KilimException {
		if (aProvider == null) {
			throw new InternalException("attempt to get the inner state of a null provider to an interface " + getQualifiedName());
		}
					
		if (providers == null) {
			throw new InternalException("attempt to get the inner state of a provider from an empty nary port " + getQualifiedName());
		}
		
		Integer result = (Integer) providers.get(aProvider);
		
		if (result == null) {
			throw new InternalException("attempt to get the inner state of an unknown provider in an nary port " + getQualifiedName() + " # " + aProvider);
		} else {
			switch(aEKind) {
				case Trigger.BIND :
					return result == BIND_TRUE;
				case Trigger.UNBIND :
					return result == UNBIND_TRUE;
				default :
					throw new InternalException("attempt to get the inner state of an unknown provider in an nary port " + getQualifiedName());
			}
		}
	}
	
	/**
	 * binds a new value source to a collection port.
	 * @param aProvider : the value source.
	 * @param jReplace : allows to rebind an already bound source, when true (this parameter is totally stupid !!!! FHO)
	 * @throws KilimException : generated when aProvider is null or when aProvider is already bound and jReplace is false.
	 */
	public void bindProvider(RuntimeSource aProvider, boolean jReplace) throws KilimException {
		if (aProvider == null) {
			throw new KilimException("attempt to bind a null provider to an interface " + getQualifiedName());
		}
					
		if (providers == null) {
			providers = new LinkedHashMap();
		}
		
		boolean cont = providers.containsKey(aProvider);	
		if (!cont) {
			providers.put (aProvider, FALSE);
		} else {
			if (jReplace) {
				providers.put(aProvider, FALSE);
			} else {
				throw new KilimException("attempt to rebind a provider to a nary interface " + getQualifiedName());
			}
		}
		
		aProvider.addInterfaceListener(this);
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#unbindProvider(RuntimeSource)
	 */
	public void unbindProvider(RuntimeSource aProvider) throws KilimException {
		if (aProvider == null) {
			throw new KilimException("attempt to unbind a null provider in the nary interface " + getQualifiedName());
		}
		if (providers == null) {
			throw new KilimException("attempt to unbind a provider from the empty interface " + getQualifiedName());
		}
		
		Object result  = providers.remove(aProvider);
		if (result == null) {
			throw new KilimException("attempt to unbind a provider not presently bound to the interface " + getQualifiedName());
		}
	
		aProvider.removeInterfaceListener(this);
	}
	
	/**
	 * returns as an iterator the bound value sources.
	 * @return Iterator
	 */
	public Iterator getBoundProviders() {
		if (providers == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return providers.keySet().iterator();
	}	

	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#bindValue(Object, RuntimeContext)
	 */
	public void bindValue(Object aValue) throws KilimException {
		throw new KilimException("attempt to get a value from a multiple value port " + getQualifiedName());
	}
		
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#unbindValue(RuntimeContext)
	 */
	public void unbindValue() throws KilimException {
		throw new KilimException("attempt to unbind a value from a multiple value port " + getQualifiedName());
	}
		
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getValue(RuntimeContext)
	 */
	public Object getValue() throws KilimException {
		throw new KilimException("attempt to get a value from a multiple value port " + getQualifiedName());
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getValue(RuntimeContext)
	 */
	
	protected Object specificGetValue() throws KilimException { return null;  }
	
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#specificBindValue(Object)
	 */
	protected void specificBindValue(Object aValue) throws KilimException { }
	
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#specificUnbindValue()
	 */
	protected void specificUnbindValue() throws KilimException { }
	
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#specificUpdate()
	 */
	public void update() throws KilimException {
		Iterator iter = getBoundProviders();
		while (iter.hasNext()) {
			ComponentInterface provider = (ComponentInterface) iter.next();
			if (provider instanceof RtCollectionPort) {
				throw new KilimException("attempt to use imbricated nary ports : " + provider.getQualifiedName() + " in " + getQualifiedName());
			} else {
				if (!triggersDone((RuntimeSource) provider, Trigger.BIND)) {
					setTriggersDone((RuntimeSource) provider, Trigger.BIND, true);
					Object resultValue = provider.getValue();
					Iterator iter2 = getTriggers(Trigger.BIND);
	 				while (iter2.hasNext()) {
						RuntimeTrigger rtElem = (RuntimeTrigger) iter2.next();
						if (rtElem.getEventKind() == Trigger.BIND) {
							Iterator iter3 = rtElem.getTransformers();
							while (iter3.hasNext()) {
								RuntimeAction rtElem3 = (RuntimeAction) iter3.next();
								rtElem3.setEventSourceValue(resultValue);
								rtElem3.execute();
								rtElem3.setEventSourceValue(null);
							}
						}
					}
				}
			}
	 	}
	}
}