/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.TemplateElementImpl;
import org.objectweb.kilim.description.Trigger;
import org.objectweb.kilim.model.instanciation.InstanciationMger;

/**
 * @author horn
 */
public abstract class RtComponentInterface extends RtComponentSource implements ComponentInterface, RuntimeSource {
	private List bTriggers;
	private List uTriggers;
	private boolean bTriggersDone;
	private boolean uTriggersDone;
	private boolean isModified = true;
	
	/**
	 * The public constructor of RtComponentInterface.
	 * @param aElement : the template description of the interface.
	 * @param aComponent :the container element in which the interface is defined.
	 */
	protected RtComponentInterface(TemplateElementImpl aElement, ContainerElement aComponent) {
		super(aElement, aComponent);
	}
	
	/**
	 * returns whether the value has been changed since the last execution of bind triggers associated to the interface.
	 * @return boolean
	 * @throws KilimException :
	 */
	public boolean isModified() throws KilimException {
		return isModified;
	}
	
	/**
	 * returns whether the triggers associated to a given kind of event have been executed or not.
	 * @param aKind : the kind of event. It is either Trigger.BIND or Trigger.UNBIND.
	 * @return boolean
	 * @throws KilimException : generated if aKind has an illegal value.
	 */
	public boolean triggersDone(int aKind) throws KilimException {
		switch(aKind) {
			case Trigger.BIND : 
				return bTriggersDone;
			case Trigger.UNBIND : 
				return uTriggersDone;
			default :
				throw new KilimException("attempt to get the trigger status in an interface " + getQualifiedName() + " for an unknow event kind : " + aKind);
		}
	}
	
	/**
	 * sets a flag indicating whether the triggers associated to a given kind of event have been performed or not.
	 * @param aKind : the kind of event. It is either Trigger.BIND or Trigger.UNBIND.
	 * @param done : true if the triggersz have been performed.
	 * @throws KilimException : generated if aKind has an illegal value.
	 */
	public void setTriggersDone(int aKind, boolean done) throws KilimException {
		switch(aKind) {
			case Trigger.BIND : 
				bTriggersDone = done;
				break;
			case Trigger.UNBIND : 
				uTriggersDone = done;
				break;
			default :
				throw new KilimException("attempt to get the trigger status in an interface " + getQualifiedName() + " for an unknow event kind : " + aKind);
		}
	}

	/**
	 * binds an interface to a runtime source.
	 * @param aProvider : the value source to be bound.
	 * @param jReplace : true if replacement is allowed.
	 * @throws KilimException : generated if aProvider is null, if the interface is already bound and jReplace is false.
	 */
	public abstract void bindProvider(RuntimeSource aProvider, boolean jReplace) throws KilimException;
	
	/**
	 * unbinds an interface from a runtime source.
	 * @param aProvider : the source value to be unbound.
	 * @throws KilimException : generated if aProvider is null or if the interface is not bound to aProvider.
	 */
	public abstract void unbindProvider(RuntimeSource aProvider) throws KilimException;	
	
	/**
	 * adds a new trigger to the interface.
	 * @param aTrigger : the trigger to be added.
	 */
	public void addTrigger(RuntimeTrigger aTrigger) {
		if (aTrigger == null) {
			return;
		}
		
		if (aTrigger.getEventKind() == Trigger.BIND) {
			if (bTriggers == null) {
				bTriggers = new ArrayList();
			}
			bTriggers.add(aTrigger);
		} else {
			if (uTriggers == null) {
				uTriggers = new ArrayList();
			}
			uTriggers.add(aTrigger);
		}
	}
	
	/**
	 * removes a trigger from an interface.
	 * @param aTrigger : the trigger to be added.
	 */
	public void removeTrigger(RuntimeTrigger aTrigger) {
		if (aTrigger == null) {
			return;
		}
		
		if (aTrigger.getEventKind() == Trigger.BIND) {
			if (bTriggers == null) {
				return;
			}
			bTriggers.remove(aTrigger);
			if (bTriggers.size() == 0) {
				bTriggers = null;
			}
		} else {
			if (uTriggers == null) {
				return;
			}
			uTriggers.remove(aTrigger);
			if (uTriggers.size() == 0) {
				uTriggers = null;
			}
		}
	}

	
	/**
	 * returns the ordered list of the triggers associated to an interface.
	 * @param aEventKind : the kind of trigger event. It should be one Trigger.BIND or Trigger.UNBIND.
	 * @return LinkedList
	 */
	public List getTriggerList(int aEventKind) {
		List list =  (aEventKind == Trigger.BIND) ? bTriggers : uTriggers;
		if (list == null) {
			return KILIM.EMPTY_LIST;
		}
		return list;
	}
	
	/**
	 * returns the as an iterator the triggers associated to an interface.
	 * @param aEventKind : the kind of trigger event. It should be one Trigger.BIND or Trigger.UNBIND.
	 * @return Iterator
	 */
	public Iterator getTriggers(int aEventKind) {
		return getTriggerList(aEventKind).iterator();
	}
		
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#setEventSource(Object)
	 */
	public void setEventSourceValue(Object aValue) throws KilimException {
		throw new KilimException("attempt to set an event source to an interface " + getQualifiedName());
	}
		
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getEventSource()
	 */
	public Object getEventSourceValue() throws KilimException {
		throw new KilimException("attempt to get an event source from an interface " + getQualifiedName());
	}

	/**
	 * @see org.objectweb.kilim.model.ComponentSource#getValue()
	 */
	
	PrintWriter pW=null;
	FileWriter fW = null;

	public Object getValue() throws KilimException {
		initializeInstanciation();
		//the local variable hasAValue indicates whether the source had a value prior to this call and thus to deterrmine whether
		//triggers should be executed or not.
		boolean hasAValue = hasValue();
		Object resultValue = specificGetValue();
		
		//if (fW == null || pW == null) {
		//	try {
		//		fW = new FileWriter("c:\\temp\\webK.log");
		//		pW = new PrintWriter(fW);
		//	} catch (Exception exc) {
		//		throw new KilimException("Problem when creating log file : c:\\temp\\webK.log");
		//	}
		//}
		
		//if ("addViewer".equals(aMethodName) && aSupport.getClass().getName().endsWith("Row")) {
		//	pW.println("addViewer sur " + aSupport);
		//	boolean pr = false;
		//	for (int i=0; i< paramObjects.length;i++) {
		//		pW.println("argument _" + i + " : " +paramObjects[i] + " (" + typeNames[i] + ")");
		//		if (paramObjects[i] != null && paramObjects[i].getClass().getName().endsWith("Cell")) pr= true;
		//	}
		//if (resultValue == null) {
		//	System.out.println("=================================================================");
		//	System.out.println("null interface " + getQualifiedName());		
		//	//new Exception().printStackTrace();
		//	System.out.println("=================================================================");
		//}
		
		if (hasAValue && !isModified) {
			return resultValue;
		}
		//triggers should not be executed here because it has already been done (in an evaluation cycle).
		if (triggersDone(Trigger.BIND)) {
			return resultValue;
		}
		
		//triggers are executed here.
		isModified = false;
		setTriggersDone(Trigger.BIND, true);		
		fireTriggers(Trigger.BIND, resultValue);
	 	return resultValue;
	}
		
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#unbindValue()
	 */
	public void unbindValue() throws KilimException {
		initializeInstanciation();
		Object aValue = getValue();
		specificUnbindValue();
		//triggers are executed here.
		isModified = true;
		if (triggersDone(Trigger.UNBIND)) {
			return;
		}
		setTriggersDone(Trigger.UNBIND, true);		
		fireTriggers(Trigger.UNBIND, aValue);
	}	
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#bindValue(Object)
	 */
	public void  bindValue(Object aValue) throws KilimException {
		if (hasValue()) {
			unbindValue();
		} else {
			initializeInstanciation();
		}
		specificBindValue(aValue);
		//triggers are executed here.
		isModified = true;
		setTriggersDone(Trigger.BIND, true);		
		fireTriggers(Trigger.BIND, aValue);
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#bindValue(Object)
	 */
	public void  update() throws KilimException {
		initializeInstanciation();
		//the local variable hasAValue indicates whether the source had a value prior to this call and thus to deterrmine whether
		//triggers should be executed or not.
		boolean hasAValue = hasValue();
		Object resultValue = specificGetValue();
		//triggers are executed here.
		isModified = false;
		setTriggersDone(Trigger.BIND, true);		
		fireTriggers(Trigger.BIND, resultValue);
	}

	protected void fireTriggers(int eventKind, Object resultValue) throws KilimException {
		//this method executes all triggers associated to the interface.
		//It therefore gets all the triggers corresponding to the eventKind.
		List lTriggers = getTriggerList(eventKind);
		if (lTriggers != null) {
			int lSize = lTriggers.size();
			for (int i = 0; i < lSize; i++) {
	 			RuntimeTrigger trig = (RuntimeTrigger) lTriggers.get(i);
				if (trig.getEventKind() == eventKind) {
					Iterator iter1 = trig.getTransformers();
					//all transformers defined in the trigger are executed.
					while (iter1.hasNext()) {
						RuntimeAction rtElem = (RuntimeAction) iter1.next();
						rtElem.setEventSourceValue(resultValue);
						rtElem.execute();
						rtElem.setEventSourceValue(null);
					}					 
				}
			}
		}
	}
	
	protected void initializeInstanciation() throws KilimException {
		Component	contain = getContainingComponent();
		if (contain == null) {
			throw new KilimException("attempt to use a source without containing component " + getLocalName());
		}

		//instanciation managers are used when an interface becomes bound in a component that is not yet instanciated.				
		if (!contain.isInitialized()) {
			//InstanciationMger mger = (InstanciationMger) contain.getController("instanciation manager");
			InstanciationMger mger = contain.getInstanciationMger();
			if (mger == null) {
				mger = contain.getInstanciationMgerFromConfiguration();
			}
			
			if (mger == null) {
				return;
			}
			
			mger.initializeInstanciation(contain, this);

			Iterator iter = contain.getSlots();
			while (iter.hasNext()) {
				ComponentSlot slot = (ComponentSlot) iter.next();
				Iterator iter0 = slot.getInterfaces();
				while (iter0.hasNext()) {
					RtComponentInterface interf = (RtComponentInterface) iter0.next();
					if (interf instanceof RtCollectionPort) {
						mger.initializePlug(contain, slot);
	 				}	
				}
			}
		}
	}
	
	/**
	 * Method specificGetValue.
	 * @return Object
	 * @throws KilimException :
	 */
	protected abstract Object specificGetValue() throws KilimException;
	
	/**
	 * Method specificBindValue.
	 * @param aValue :
	 * @throws KilimException :
	 */
	protected abstract void specificBindValue(Object aValue) throws KilimException;
	
	/**
	 * Method specificUnbindValue.
	 * @throws KilimException :
	 */
	protected abstract void specificUnbindValue() throws KilimException;
}