/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;
import java.util.Stack;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.NamedElement;
import org.objectweb.kilim.description.Property;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.description.TemplateElementImpl;

/**
 * @author horn
 */

public class RtComponentProperty extends RtComponentInterface implements ComponentProperty {
	private static Object UNSET = new Object();
	private boolean used;
	private Object value;
	
		
	/**
	 * The public constructor for RtComponentProperties.
	 * @param aElement : the template description of the property.
	 * @param aComponent : the component containing the property.
	 */
	public RtComponentProperty(TemplateElementImpl aElement, Component aComponent) {
		super(aElement, aComponent);
		value = ((Property) getElementDescription()).getValue();
	}

	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 */
	public String getLocalName() {
		NamedElement elem = (NamedElement) getElementDescription();
		return elem.getLocalName();
	}
	
	/**
	 * sets the value of a property. It is strictly equivalent to bindValue(aValue).
	 * @param aValue : the value to be set.
	 * @throws KilimException : generated if the value is illegal (type mismatch, for example).
	 */
	public void setValue(Object aValue) throws KilimException {
		bindValue(aValue);
	}
		
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getEventSource()
	 */
	public Object getEventSourceValue() throws KilimException {
		return null;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#setEventSource(Object)
	 */
	public void setEventSourceValue(Object aSource) throws KilimException { }
	
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#bindProvider(RuntimeSource, boolean)
	 */
	public void bindProvider(RuntimeSource aSource, boolean jReplace) throws KilimException {
		TemplateDescription tmp0 = getContainingComponent().getTemplate();
		String lName = getLocalName();
		String tmpList = tmp0.getName();
		while ((tmp0 = tmp0.getSuperTemplate()) != null) {
			Iterator iter = tmp0.getBindings(lName, true);
			if (iter != KILIM.EMPTY_ITERATOR) {
				tmpList = tmpList + ", " + tmp0.getName();
			}
		}
		throw new KilimException("attempt to bind a property " + getQualifiedName() + " (binding definition in templates  : " + tmpList + ")");
	}
	
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#unbindProvider(RuntimeSource)
	 */
	public void unbindProvider(RuntimeSource aSource) throws KilimException {
		throw new KilimException("attempt to unbind a provider " + aSource + " from a property " + getQualifiedName());
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getValue(RuntimeContext) 
	 */
	protected Object specificGetValue() throws KilimException {
		Object resultValue = null;
		if (mappingContext != null) {
			mappingContext.getCallStack().push(this);
		}
		if (value == UNSET) {
			throw new KilimException("attempt to get the value of an unset property " + getQualifiedName());
		}

		mapper.enterContext(mappingContext);
		resultValue = mapper.getPropertyValue(value, mappingContext);
		if (mappingContext != null) {
			mappingContext.getCallStack().pop();
		}
		mapper.leaveContext(mappingContext);
		used = true;
		return resultValue;
	}

	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#unbindValue()
	 */
	protected void specificUnbindValue() throws KilimException {
		value = UNSET;
	}	
		
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#bindValue(Object)
	 */
	protected void  specificBindValue(Object aValue) throws KilimException {
		value = aValue;
	}
		
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isSingleValueInterface()
	 */
	public boolean isSingleValuePort() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isCollectionInterface()
	 */
	public boolean isCollectionPort() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isProvider()
	 */
	public boolean isProvider() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isProperty()
	 */
	public boolean isProperty() {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#checkValue()
	 */
	public boolean hasValue() {
		return value != UNSET;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#checkValue(Stack)
	 */
	public boolean checkValue(Stack exclude) throws KilimException {
		return true;
	}
}	
