/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.objectweb.kilim.KilimConfiguration;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.TemplateElementImpl;
import org.objectweb.kilim.model.mapping.Mapper;
import org.objectweb.kilim.model.mapping.MappingContext;

/**
 * @author horn
 */

public abstract class RtComponentSource extends RtComponentElement  implements RuntimeSource {
	//svpStack and knownValues are two data structures necessary to solve the "evaluation cycle" problem
	//svpStack is the stack of the single value ports which are in the process of being evaluated.
	//knownValues is a hashmap containing the value of already evaluated single value ports.
	protected static Stack svpStack = new Stack();
	protected static HashMap knownValues = new HashMap();
	
	protected Mapper mapper;
	protected MappingContext mappingContext;

	//nAryListeners are mainly used to implement the plug operations	
	private List nAryListeners;

	protected RtComponentSource(TemplateElementImpl aElement, ContainerElement aComponent) {
			super(aElement, aComponent);
			mapper = KilimConfiguration.getMappingStrategy().getDefaultMapper();
			mappingContext = KilimConfiguration.getMappingContext();
	}
	
	/**
	 * Method getCurrentSVP.
	 * @return RtSingleValuePort
	 */
	public static RtSingleValuePort getCurrentSVP()  {
		int svpSize = svpStack.size();
		if (svpSize == 0) {
			return null;
		} else {
			return (RtSingleValuePort) svpStack.get(svpSize - 1);
		}
	}	
	
	/**
	 * Method evaluationPerformed.
	 * @param aPort : a single value port.
	 * @return boolean : the value is true if the single value port has already been evaluated. False otherwise.
	 */
	public static boolean evaluationPerformed(RtSingleValuePort aPort) {
		if (knownValues.containsKey(aPort)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#addInterfaceListener(RtCollectionInterface)
	 */
	public void addInterfaceListener(RtCollectionPort aInterface) throws KilimException {
		if (aInterface == null) {
			throw new KilimException("attempt to add a null nary port listener to an interface " + getQualifiedName());
		}
			
		if (nAryListeners == null) {
			nAryListeners = new ArrayList();
		}
		
		if (nAryListeners.contains(aInterface)) {
			throw new KilimException("attempt to add an already known nary port listener to an interface " + getQualifiedName());
		}			
		nAryListeners.add(aInterface);
	}	
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#removeInterfaceListener(RtCollectionInterface)
	 */
	public void removeInterfaceListener(RtCollectionPort aInterface) throws KilimException {
		if (aInterface == null) {
			throw new KilimException("attempt to remove a null nary port listener from an interface " + getQualifiedName());
		}			
		if (nAryListeners == null) {
			throw new KilimException("attempt to remove a nary port listener from an interface " + getQualifiedName() + " without listeners");
		}
		
		boolean result = nAryListeners.remove(aInterface);
		if (!result) {
			throw new KilimException("attempt to remove a unknown nary port listener from an interface " + getQualifiedName());
		}
	}
	
	/**
	 * returns as an iterator the interface listeners associated to the source.
	 * @return Iterator
	 */
	public Iterator getInterfaceListeners() {
		if (nAryListeners == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		
		return nAryListeners.iterator();
	}
}
