/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;
import java.util.Stack;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.NamedElement;
import org.objectweb.kilim.description.Port;
import org.objectweb.kilim.description.TemplateDescription;

/**
 * @author horn
 */
public class RtSingleValuePort extends RtComponentInterface implements ComponentInterface {	
	//state fields 
	private RuntimeSource provider;
	private Object resultValue;
	private boolean gotAValue;
	
	//fields required for displaying error messages in case of multiple bindings on the unary port.
	private boolean firstCall = true;
	private int nbBound;
	private boolean msgDisplayed;

	/**
	 * The public constructor for run time descriptors of single value ports. 
	 * @param aPort : the template port descriptor
	 * @param aContainer : either the containing component or slot
	 */
	public RtSingleValuePort(Port aPort, ContainerElement aContainer) {
		super(aPort, aContainer);
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#hasValue()
	 */
	public boolean hasValue() {
		return gotAValue;
	}
	
	/**
	 * Method getBufferedValue : returns the current buffered value. This method returns null when invoked prior to the evaluation.
	 * @return Object
	 */
	public Object getBufferedValue() {
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isSingleValueInterface()
	 */
	public boolean isSingleValuePort() {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isCollectionInterface()
	 */
	public boolean isCollectionPort() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isProvider()
	 */
	public boolean isProvider() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#isProperty()
	 */
	public boolean isProperty() {
		return false;
	}

	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 */
	public String getLocalName() {
		NamedElement elem = (NamedElement) getElementDescription();
		return elem.getLocalName();
	}
		
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#bindProvider(RuntimeSource, boolean)
	 */
	public void bindProvider(RuntimeSource aProvider, boolean jReplace) throws KilimException {
		//The Kilim2 runtime does not accept bindings to null providers. One should use either a RTNullElement or 
		//a RTExternalValue whose value is null.
		if (aProvider == null) {
			String tmpName = getContainingComponent().getTemplate().getName();
			throw new KilimException("attempt to bind a null provider to an unary port " + getQualifiedName() + "(template : " + tmpName + ")");
		}

		//detection of illegal multiple binding on unary ports.		
		provider = aProvider;
		nbBound++;
				
		if (nbBound > 1 && !jReplace) {
			if (!msgDisplayed) {
				msgDisplayed = true;
				TemplateDescription tmp0 = getElementDescription().getContainingTemplate();
				String lName = getLocalName();
				String tmpList = "";
				do {
					Iterator iter = tmp0.getBindings(lName, true);
					if (iter != KILIM.EMPTY_ITERATOR) {
						tmpList = tmpList + "\"" + tmp0.getName() + "\" ";
					}
				} while ((tmp0 = tmp0.getSuperTemplate()) != null);
				
				TemplateDescription tmp1 = getContainingComponent().getTemplate();
				
				if (tmp1 != tmp0) {
					do {
						Iterator iter = tmp1.getBindings(lName, true);
						if (iter != KILIM.EMPTY_ITERATOR) {
							tmpList = tmpList + " : " + tmp1.getName();
						}
					} while ((tmp1 = tmp1.getSuperTemplate()) != null);
				}
			}
						
			//System.err.println("warning : rebinding a unary interface " + getQualifiedName());
		}
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#unbindProvider(RuntimeSource)
	 */
	public void unbindProvider(RuntimeSource aProvider) throws KilimException {
		if (provider == null) {
			throw new KilimException("attempt to unbind a provider from the empty interface " + getQualifiedName());
		}
		
		//unbinding a provider does not remove the previous buffered value (if any).
		provider = null;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getValue()
	 */
	protected Object specificGetValue() throws KilimException {	
		if (gotAValue) {
			return resultValue;
		}
		
		svpStack.push(this);
			
		//It is legal in Kilim2 grammar to have "unprovided ports" i.e. ports having no declared explicit or inlined provider.
		//Kilim2 runtime handles them as ports bound to a external reference whose value is null.
		if (provider == null) {
			provider = new RtExternalValue(getLocalName() + KILIM.getUniqueIndex(), null);
		} else {
			resultValue = ((RuntimeSource) provider).getValue();
			gotAValue = true;
		}
		svpStack.pop();
		return resultValue;
	}
		
	/**
	 * @see org.objectweb.kilim.model.ComponentInterface#bindValue(Object, RuntimeContext)
	 */
	protected void specificBindValue(Object aValue) throws KilimException {
		if (provider != null) {
			throw new KilimException("attempt to bind a value to a unary port " + getQualifiedName() + " having a provider " + provider);
		}	
		resultValue = aValue;
		gotAValue = true;
	}
			
	/**
	 * @see org.objectweb.kilim.model.RtComponentInterface#specificUnbindValue()
	 */
	protected void specificUnbindValue() throws KilimException {
		if (!gotAValue) {		
			throw new KilimException("attempt to unbind a Value from a port " + getQualifiedName() + " without value");
		}		
		resultValue = null;
		gotAValue = false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#checkValue(Stack)
	 */
	public boolean checkValue(Stack exclude) throws KilimException {
		if (hasValue()) {
			return true;
		}
				
		int eSize = exclude.size();
		for (int i = 0; i < eSize; i++) {
			if (this == exclude.get(i)) {
				return false;
			}
		}
		
		if (provider == null) {
			return false;
		}

		return provider.checkValue(exclude);
	}
}