/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.instanciation;

import java.util.HashMap;

import org.objectweb.kilim.KilimException;

/**
 * This class is a simple container dedicated to the definition of instanciation strategies. It is used by the component
 * factory to find the instanciation manager to be associated to each application component. It is possible to redefine
 * a default instanciation manager, and to define per template managers and per instance managers. 
 * The factory uses the following look up strategy. The search is stopped as soon as a manager is found.
 * the component factory first looks for a manager explicitely associated to the component.
 * If none can be found, it then looks for a manager explicitely associated to the component  template.
 * If none can be found, it recursively looks for a manager explicitely associated to super templates.
 * If none can be found it looks for a default manager.
 * 
 * No null value can be returned, since this class defines a NAryPreInstanciationMger as default instanciation manager
 * and no null value is accepted by the setDefaultMger().
 * @author horn
 */

public class DefaultInstanciationStrategy implements InstanciationStrategy {
	private InstanciationMger defaultMger = new BDUInstanciationMger();
	private HashMap perTemplateMger;
	private HashMap perInstanceMger;
	
	/**
	 * @see java.lang.Object#Object()
	 */
	public DefaultInstanciationStrategy() throws KilimException {
	}
	
	/**
	 * Method InstanciationStrategy.
	 * @param aMger : a default instanciation manager (this ctor just contains a call to setDefaultManager(aMger)
	 * @throws KilimException : generated if the reference is null. A NullInstanciationMger should be indicated if
	 *  no action should be performed at instanciation time.
	 */
	public DefaultInstanciationStrategy(InstanciationMger aMger) throws KilimException {
		setDefaultMger(aMger);
	}

	/**
	 * @see org.objectweb.kilim.model.services.InstanciationStrategy#setDefaultMger(InstanciationMger)
	 */
	public void setDefaultMger(InstanciationMger aMger) throws KilimException {
		if (aMger == null) {
			throw new KilimException("attempt to install a null instanciation manager as default");
		}
		defaultMger = aMger;
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.InstanciationStrategy#getDefaultMger()
	 */
	public InstanciationMger getDefaultMger() {
		return defaultMger;
	}

	/**
	 * @see org.objectweb.kilim.model.services.InstanciationStrategy#setPerTemplateMger(String, InstanciationMger)
	 */
	public void setPerTemplateMger(String aName, InstanciationMger aManager) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to install a manager for template through a null name");
		}

		if (perTemplateMger == null) {
			perTemplateMger = new HashMap();
		}
		perTemplateMger.put(aName, aManager);
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.InstanciationStrategy#getPerTemplateMger(String)
	 */
	public InstanciationMger getPerTemplateMger(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get a template manager through a null name ");
		}
		
		if (perTemplateMger == null) {
			return null;
		}
		return (InstanciationMger) perTemplateMger.get(aName);
	}

	/**
	 * @see org.objectweb.kilim.model.services.InstanciationStrategy#setPerInstanceMger(String, InstanciationMger)
	 */
	public void setPerInstanceMger(String aName, InstanciationMger aManager) throws KilimException {
		if (aManager == null) {
			throw new KilimException("attempt to install a null instanciation manager for template " + aName);
		}

		if (perInstanceMger == null) {
			perInstanceMger = new HashMap();
		}
		perInstanceMger.put(aName, aManager);
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.InstanciationStrategy#getPerInstanceMger(String)
	 */
	public InstanciationMger getPerInstanceMger(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get a instance manager through a null name ");
		}
		
		if (perInstanceMger == null) {
			return null;
		}
		return (InstanciationMger) perInstanceMger.get(aName);
	}
}
