/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.instanciation;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.ComponentSlot;
import org.objectweb.kilim.model.mapping.MappingContext;

/**
 * @author horn
 * All Kilim components and slots have a state which is either UNINITIALIZED or INITIALIZED.  A component and its slots are both created by the component factory 
 * in the UNINITIALIZED state. A component interface value can only be accessed and used when the component and slot the interface belongs to are in the INITIALIZED state. 
 * It is the responsability of instanciation managers to "initialize" components and slots. The Kilim model therefore uses pre and post methods :
 * the underlying logics for interfaces  is roughly the following :
 * if (containing component or slot in UNITIALIZED state) {
 * 	initializeInstanciation(component, interface);
 * 	access to the interface (i.e. invoke getValue() on the interface).
 * 	finalizeInstanciation(component, interface);
 * }
 *  /////////////////////////////////////////////////////////////////////////////////
 * The same constraints apply to the plug operation. The underlying logics for interfaces  is roughly the following :
 * if (component to be plugged or slot in UNITIALIZED state) {
 * 	initializePlug(component, slot);
 * 	plug the component
 * 	finalizePlugcomponent, slot);
 * }
 */
	
public abstract class InstanciationMger {

	/**
	 * This is the "pre" method called by the Kilim runtime when it has to access the value of an interface (port, provider, property) belonging to a UNITIALIZED component or slot.
	 * This method is strictly equivalent to initializeInstanciation(aComponent, aInterface, null);
	 * @param aComponent : the component containing the interface
	 * @param aInterface : the accessed interface
	 * @throws KilimException : generated by the instantiation manager in case of problems. These are specific to each manager.
	 */
	public void initializeInstanciation(Component aComponent, ComponentInterface aInterface) throws KilimException {
		initializeInstanciation(aComponent, aInterface, null);
	}

	/**
	 * This is the "pre" method called by the Kilim runtime when it has to get the value of an interface (port, provider, property) belonging to a UNITIALIZED component or slot.
	 * @param aComponent : the component containing the interface
	 * @param aInterface : the accessed interface
	 * @param aContext : the mapping context
	 * @throws KilimException : generated by the instantiation manager in case of problems. These are specific to each manager.
	 */
	public abstract void initializeInstanciation(Component aComponent, ComponentInterface aInterface, MappingContext aContext) throws KilimException;	
	
	/**
	 * This is the "post" method called by the Kilim runtime when it has to get the value of an interface (port, provider, property) belonging to a UNITIALIZED component or slot.
	 * This method is strictly equivalent to finalizeInstanciation(aComponent, aInterface, null);
	 * @param aComponent : the component containing the interface
	 * @param aInterface : the accessed interface
	 * @throws KilimException : generated by the instantiation manager in case of problems. These are specific to each manager.
	 */
	public void finalizeInstanciation(Component aComponent, ComponentInterface aInterface) throws KilimException {
		finalizeInstanciation(aComponent, aInterface, null);
	}
	
	/**
	 * This is the "post" method called by the Kilim runtime when it has to get the value of an interface (port, provider, property) belonging to a UNITIALIZED component or slot.
	 * @param aComponent : the component containing the interface
	 * @param aInterface : the accessed interface
	 * @param aContext : the mapping context
	 * @throws KilimException : generated by the instantiation manager in case of problems. These are specific to each manager.
	 */
	public abstract void finalizeInstanciation(Component aComponent, ComponentInterface aInterface, MappingContext aContext) throws KilimException;		

	/**
	 * This is the "pre" method called by the Kilim runtime when it has to plug either an UNINITIALIZED component or an INITIALIZED component into an UNITIALIZED slot.
	 * @param aComponent : the component to be plugged
	 * @param aSlot : the slot
	 * @throws KilimException : generated by the instantiation manager in case of problems. These are specific to each manager.
	 */
	public void initializePlug(Component aComponent, ComponentSlot aSlot) throws KilimException {
		initializePlug(aComponent, aSlot, null);
	}

	/**
	 * This is the "pre" method called by the Kilim runtime when it has to plug either an UNINITIALIZED component or an INITIALIZED component into an UNITIALIZED slot.
	 * @param aComponent : the component to be plugged
	 * @param aSlot : the slot
	 * @param aContext : the maping context
	 * @throws KilimException : generated by the instantiation manager in case of problems. These are specific to each manager.
	 */
	public abstract void initializePlug(Component aComponent, ComponentSlot aSlot, MappingContext aContext) throws KilimException;	
	
	/**
	 * This is the "post" method called by the Kilim runtime when it has to plug either an UNINITIALIZED component or an INITIALIZED component into an UNITIALIZED slot.
	 * @param aComponent : the component to be plugged
	 * @param aSlot : the slot
	 * @throws KilimException : generated by the instantiation manager in case of problems. These are specific to each manager.
	 */
	public void finalizePlug(Component aComponent, ComponentSlot aSlot) throws KilimException {
		finalizePlug(aComponent, aSlot, null);
	}
	
	/**
	 * This is the "post" method called by the Kilim runtime when it has to plug either an UNINITIALIZED component or an INITIALIZED component into an UNITIALIZED slot.
	 * @param aComponent : the component to be plugged
	 * @param aSlot : the slot
	 * @param aContext : the maping context
	 * @throws KilimException : generated by the instantiation manager in case of problems. These are specific to each manager.
	 */
	public abstract void finalizePlug(Component aComponent, ComponentSlot aSlot, MappingContext aContext) throws KilimException;		
}

