/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.mapping;

import org.objectweb.kilim.KilimException;

/**
 * This class is a simple container dedicated to the definition of mapping strategies. It is used at run time
 *  to find the mapper to be associated to each application component. It is possible to redefine
 * a default  mapper, and to define per template mappers and per instance mappers. 
 * The run time uses the following look up strategy. The search is stopped as soon as a mapper is found.
 * the runtime first looks for a mapper explicitely associated to the component.
 * If none can be found, it then looks for a mapper explicitely associated to the component  template.
 * If none can be found, it recursively looks for a mapper explicitely associated to super templates.
 * If none can be found it looks for a default mapper.
 * 
 * No null value can be returned, since this class defines a default  mapper and no null value is accepted by the 
 * setDefaultMapper() method
 * @author horn
 */

public interface MappingStrategy {
	
	/**
	 * Method setDefaultMapper.
	 * @param aMapper : a default mapper (this ctor just contains a call to setDefaultManager(aMapper). This value
	 * cannot be null. A NullMapper should be used if no action should be performed at instanciation time.
	 * @throws KilimException : generated if the reference is null. 
	 */
	void setDefaultMapper(Mapper aMapper) throws KilimException;
	
	/**
	 * Method getDefaultMapper returns the default mapper.
	 * @return Mapper
	 */
	Mapper getDefaultMapper();

	/**
	 * Method setPerTemplateMapper sets the mapper to be used when the component is an instance of a template or one
	 * of its subtemplates.
	 * @param aName is name of the template.
	 * @param aMapper is the mapper to be used.
	 * @throws KilimException : generated when aName is null.
	 */
	void setPerTemplateMapper(String aName, Mapper aMapper) throws KilimException;
	
	/**
	 * Method getPerTemplateMapper
	 * @param aName : the name of the template.
	 * @return Mapper
	 * @throws KilimException : generated when aName is null
	 */
	Mapper getPerTemplateMapper(String aName) throws KilimException;
	
	/**
	 * Method setPerInstanceMger sets the mapper to be used for a given instance.
	 * @param aName : the name of the component.
	 * @param aMapper : the mapper to be used
	 * @throws KilimException : generated when aName is null.
	 */
	void setPerInstanceMapper(String aName, Mapper aMapper) throws KilimException;
	
	/**
	 * Method getPerInstanceMapper.
	 * @param aName : the name of the component
	 * @return Mapper :
	 * @throws KilimException : generated when aName is null.
	 */
	Mapper getPerInstanceMapper(String aName) throws KilimException;
}
