/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.services;

import java.util.HashMap;
import java.util.Iterator;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentElement;
import org.objectweb.kilim.model.RtExternalValue;

/**
 * @author horn
 */
public class DefaultNamingContext implements NamingContext {	
	private String localName;	
	private NamingContext parentContext;
	private HashMap childContexts;
	private HashMap boundNames;
	private ExternalValueReferences externalNames;
			
	/**
	 * a public constructor for a naming context.
	 * @param aName : the local name of the naming context.
	 * @param aParent : the parent naming context.
	 * @param extNames : a table of external references (instanciateds objects not associated to a kilim representation).
	 * @throws KilimException : generated if aName is null.
	 */
	public DefaultNamingContext(String aName, NamingContext aParent, ExternalValueReferences extNames) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to create a naming context with a null name");
		}
		localName = aName;
		parentContext = aParent;
		externalNames = extNames;
	}
	
	/** a public constructor for a naming context. It is strictly equivalent to DefaultNamingContext(aName, aParent, null).
	 * @param aName : the local name of the naming context.
	 * @param aParent : the parent naming context.
	 * @throws KilimException :: generated if aName is null.
	 */
	public DefaultNamingContext(String aName, NamingContext aParent) throws KilimException {
		this(aName, aParent, null);
	}	
	
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#getParentNamingContext()
	 */
	public NamingContext getParentNamingContext() {
		return parentContext;
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#addChildNamingContext(String, NamingContext)
	 */
	public void addChildNamingContext(String aName, NamingContext aContext) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to add a child context with a null name in naming context " + getQualifiedName());
		}
		if  (aContext == null) {
			throw new KilimException("attempt to add a null child context " + aName + " in naming context " + getQualifiedName());
		}
		if (childContexts == null) {
			childContexts = new HashMap();
		}	
		if	(childContexts.containsKey(aName)) {
			throw new KilimException("attempt to add a child context with an already existing name " + aName + " in naming context " + getQualifiedName());
		}
		childContexts.put(aName, aContext);
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#removeChildNamingContext(String)
	 */
	public void removeChildNamingContext(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to remove a child context through a null name in naming context " + getQualifiedName());
		}
		if (childContexts == null) {
			throw new KilimException("attempt to remove a child context from an empty naming context " + getQualifiedName());
		}
		Object result = childContexts.remove(aName);
		if (result == null) {
				throw new KilimException("attempt to remove an unknown child context " + aName + " in naming context " + getQualifiedName());
		}
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#getChildNamingContexts()
	 */
	public Iterator getChildNamingContexts() {
		if (childContexts == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return childContexts.keySet().iterator();
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#setParentNamingContext(NamingContext)
	 */
	public void setParentNamingContext(NamingContext aContext) {
		parentContext = aContext;
	}

	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#setExternalReferenceMap(HashMap)
	 */
	public void setExternalReferences(ExternalValueReferences xReferences) {
		externalNames = xReferences;
	}

	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#getExternalReferenceMap()
	 */
	public ExternalValueReferences getExternalReferenceMap() {
		return externalNames;
	}
		
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#getQualifiedName()
	 */
	public String getQualifiedName() {
		if (parentContext == null) {
			return localName;
		}
		
		String contName = parentContext.getQualifiedName();

		if ("".equals(contName)) {
			return localName;
		}
		return contName + "/" + localName;
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#getLocalName()
	 */
	public String getLocalName() {
		return localName;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
			return "[" + getQualifiedName() + "]";
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#addBoundName(String, Object)
	 */
	public void addBoundName(String aName, ComponentElement aElement) throws KilimException {
		if (aName == null) {
			throw  new KilimException("null name in addBoundName of naming context " + getQualifiedName());
		}
		
		if (aElement == null) {
			throw  new KilimException("null element in addBoundName of naming context " + getQualifiedName());
		}
		
		if (boundNames == null) {
			boundNames = new HashMap();
		}
		
		if (boundNames.containsKey(aName)) {
			throw  new KilimException("Name clash in addBoundName of naming context " + getQualifiedName() + " : \"" + aName + "\"");
		}
	
		boundNames.put(aName, aElement);
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#removeBoundName(String)
	 */
	public void removeBoundName(String aName) throws KilimException {
		if (aName == null) {
			throw  new KilimException("null name in removeBoundName of naming context " + getQualifiedName());
		}
		
		if (boundNames == null) {
			throw  new KilimException("no bound names in current naming context " + getQualifiedName());
		}
		
		Object previous = boundNames.remove(aName);
		if (previous == null) {	
			throw  new KilimException("name " + aName + " unknown in current naming context " + getQualifiedName());
		}		
	}
		
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#getBoundNames()
	 */
	public Iterator getBoundNames() {
		if (boundNames == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		
		return boundNames.keySet().iterator();
	}
		
	/**
	 * @see org.objectweb.kilim.model.services.NamingContext#resolveReference(String, Component)
	 */
	public ComponentElement resolveReference(String aName, Component origin) throws KilimException {
		// A / prefix in a name is assumed to be a convention for designating external references

		if (aName.startsWith("/")) {	
			if (externalNames != null) {				
				Object eResult = externalNames.getExternalValueReference(aName);
				if (eResult != null) {
					RtExternalValue result =  new RtExternalValue(aName, eResult);
					boundNames.put(aName, result);
					return result;
				}
			}			
			throw  new KilimException("unknown external reference " + aName + " in naming context " + getQualifiedName() + " from component " + origin.getQualifiedName());
		}
		
		// A ../ prefix is a convention for performing the lookup in the super component (here the super name service).
		//When the super name service does not exist, the name is assumed to reference an external value.

		if (aName.startsWith("../")) {
			NamingContext parent = getParentNamingContext();
			if (parent == null) {
				if (externalNames != null) {
					Object result = externalNames.getExternalValueReference(aName);
					if (result != null) {
						//return new RtExternalValue(aName, result, fMapper, fMappingContext);
						return new RtExternalValue(aName, result);
					}
				}
				throw  new KilimException("unresolvable reference (1) " + aName + " in naming context " + getQualifiedName() + " from component " + origin.getQualifiedName());
			}
			return parent.resolveReference(aName.substring(3), origin);
		}
		
		// A ./ prefix is a convention for doing the lookup in the current component (here the current factory).It is just stripped.
		if (aName.startsWith("./")) {
			aName = aName.substring(2);
		}
		
		// The lookup is performed here. This allows to deal with ad hoc local naming spaces : the corresponding names should 
		// have the form "name1/name2". Any kind of convention may however be used, since no check is performed.
		// The look up is done through the "boundNames" table.
		if (boundNames != null) {
			ComponentElement result = (ComponentElement) boundNames.get(aName);
			if (result != null) {
				return result;
			}
		}
		
		// the existence of a / in the name means that the look up should be done in a subcomponent. If the subcomponent
		// does not exist, it is assumed the name references an external value.This allows the use of a predefinite "context"
		// in which all external references are defined (such as _EXTERNAL/extern_edit = ....., _EXTERNAL/extern_debug = ....
		int pos = aName.indexOf("/");
		if (pos != -1) {
			String lName = aName.substring(0, pos);
			Object child = null;
			if (childContexts != null) {
				child = childContexts.get(lName);
			}
			if (child == null) {
				if (externalNames != null) {
					Object result = externalNames.getExternalValueReference(aName);
					if (result != null) {
						//return new RtExternalValue(aName, result, fMapper, fMappingContext);
						return new RtExternalValue(aName, result);
					}
				}
				throw  new KilimException("unresolvable reference (2) " + aName + " in naming context " + getQualifiedName() + " from component " + origin.getQualifiedName());
			}
			
			if (!(child instanceof NamingContext)) {
				throw  new KilimException("attempt to use as a naming context an illegal element " + lName + " in naming context " + getQualifiedName() + " from component " + origin.getQualifiedName());
			}
			return ((NamingContext) child).resolveReference(aName.substring(pos + 1), origin);
		}
				
		// the previous lookup did not find the name in the "boundNames" table. The name is assumed to be an external reference
		if (externalNames != null) {
			Object result = externalNames.getExternalValueReference(aName);
			if (result != null) {
				//return new RtExternalValue(aName, result, fMapper, fMappingContext);
				return new RtExternalValue(aName, result);
			}
		}
		return null;
	}
}