/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Properties;

import org.objectweb.kilim.description.BasicNamedElement;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.Property;
import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.TemplateDescription;

/**
 * Implementation of Repository using a ResourceLoader and a kilim xml format Parser.
 * {@see org.objectweb.kilim.repository.Repository}
 * @author dutoo, horn
 */
public class ResourceRepository1 extends  ResourceRepository {
	
	/**
	 * Creates a new ResourceRepository with the given kilim xml format Parser and its ClassLoader as resource loader.
	 */
	public ResourceRepository1() {
		super();
	}
	
	/**
	 * Creates a new ResourceRepository with the given kilim xml format Parser and its ClassLoader as resource loader.
	 * @param aParser the kilim xml format parser to be used
	 */
	public ResourceRepository1(TemplateDescriptionParser aParser) {
		super(aParser); 
	}
		
	/**
	 * Method getSingleDescription. This method parses a single ".kilim" file. It updates the ResourceMapping object
	 * received as a parameter by adding new unknown templates, new "super" references, etc ...
	 * @param resourceName : the name of the template to be parsed
	 * @param parsingResult : the ResourceMapping object used to store the parsing results to be used during next iterations.
	 * @return TemplateDescription : the internal representation of the template described in the resource.
	 * @throws ResourceNotFoundException
	 */
	protected TemplateDescription getSingleDescription(String resourceName, ResourceMapping parsingResult) throws ResourceNotFoundException {
		TemplateDescription toBeReturned = super.getSingleDescription(resourceName, parsingResult);
		
		String prp_fl_nm = resourceName + ".prop";
		
		InputStream is = null;
		try {
			is = getResourceLoader().getResource(prp_fl_nm);
		} catch (ResourceNotFoundException rsnf) {
			System.err.println("warning : No Properties found for " + prp_fl_nm + ".");
		}
		
		if (is == null) { System.err.println("warning : No .prop file found for " + prp_fl_nm + "."); }
		
		Properties addtnl_prps = new Properties();
		try {
			if (is != null) { addtnl_prps.load(is); }
		} catch (IOException io) {

			System.err.println("warning : Properties " + prp_fl_nm + "could not be loaded properly.");
		}
		
		for (Enumeration it = addtnl_prps.keys() ; it.hasMoreElements() ;) {
			String crrt_prp = (String) it.nextElement();
			BasicNamedElement prp = null;
			try {
				prp = toBeReturned.getAnyProvider(crrt_prp , false);
			} catch (KilimException exc) {
				System.err.println("warning : attempt to set an unknown property " + crrt_prp);
			}
			
			if (prp == null) {
				System.err.println("warning : trying to set an unknown property " + crrt_prp);
			} else  if (!(prp instanceof Property)) {
				System.err.println("warning : attempt to set a provider which is not a property " + crrt_prp + " (" + prp.getClass() + ")");
			}

			if (prp != null) {
				setValue((Property) prp , (String) addtnl_prps.get(crrt_prp));
			}
		}
				
		return toBeReturned;
	}
	
	private static void setValue(Property property, String value) {
		
		int tp_knd = property.getTypeKind();		
		
		switch (tp_knd) {
		
			case KILIM.BOOLEAN:
				try {
					Boolean bln = new Boolean(value);
					property.setValue(bln);
				} catch (Exception e) {
					error(property , value , e);
					e.printStackTrace();
				}
			break;
			
			case KILIM.CHAR:
				try {
					Character chr = new Character(value.charAt(0));
					property.setValue(chr);
				} catch (Exception e) {
					error(property, value, e);
					e.printStackTrace();
				}
			break;
			
			case KILIM.BYTE:
				try {
					Byte bt = new Byte(value);
					property.setValue(bt);
				} catch (Exception e) {
					error(property , value , e);
					e.printStackTrace();
				}
			break;
			
			case KILIM.SHORT:
				try {
					Short shrt = new Short(value);
					property.setValue(shrt);
				} catch (Exception e) {
					error(property , value , e);
					e.printStackTrace();
				}
			break;
			
			case KILIM.INT:
				try {
					Integer intgr = new Integer(value);
					property.setValue(intgr);
				} catch (Exception e) {
					error(property , value , e);
					e.printStackTrace();
				}
			break;
			
			case KILIM.LONG:
				try {
					Long lng = new Long(value);
					property.setValue(lng);
				} catch (Exception e) {
					error(property, value, e);
					e.printStackTrace();
				}
			break;
			
			case KILIM.FLOAT:
				try {
					Float flt = new Float(value);
					property.setValue(flt);
				} catch (Exception e) {
					error(property , value , e);
					e.printStackTrace();
				}
			break;
			
			case KILIM.DOUBLE:
				try {
					Double dbl = new Double(value);
					property.setValue(dbl);
				} catch (Exception e) {
					error(property, value, e);
					e.printStackTrace();
				}
			break;

			case KILIM.OBJECT:
			case KILIM.STRING:
				try {
					property.setValue(value);
				} catch (Exception e) {
					error(property , value , e);
					e.printStackTrace();
				}
			break;
			
			default:
				throw new InternalException();
		
		}
	}
	
	private static void error(Property property, String value, Exception exc) {
		System.err.println("warning : " + property.getType() + " Property " + property.getLocalName()
							+ " could not be overloaded with provided value " + value + " because of the following exception :");
		exc.printStackTrace();
	}
}